@@ -3,7 +3,7 @@
 *                                                                      *
 * This file is part of the FINAL CUT widget toolkit                    *
 *                                                                      *
-* Copyright 2013-2021 Markus Gans                                      *
+* Copyright 2013-2023 Markus Gans                                      *
 *                                                                      *
 * FINAL CUT is free software; you can redistribute it and/or modify    *
 * it under the terms of the GNU Lesser General Public License as       *
@@ -65,8 +65,9 @@ FWidget*  FWidget::main_widget          {nullptr};  // main application widget
 FWidget*  FWidget::active_window        {nullptr};  // the active window
 FWidget*  FWidget::focus_widget         {nullptr};  // has keyboard input focus
 FWidget*  FWidget::clicked_widget       {nullptr};  // is focused by click
+FWidget*  FApplication::clicked_widget  {nullptr};  // is focused by click
 FWidget*  FWidget::open_menu            {nullptr};  // currently open menu
-FWidget*  FWidget::move_size_widget     {nullptr};  // move/size by keyboard
+FWidget*  FWidget::move_resize_widget   {nullptr};  // move or resize by keyboard
 FWidget*  FApplication::keyboard_widget {nullptr};  // has the keyboard focus
 int       FApplication::loop_level      {0};        // event loop level
 int       FApplication::quit_code       {EXIT_SUCCESS};
@@ -74,6 +75,10 @@ bool      FApplication::quit_now        {false};
 uInt64    FApplication::next_event_wait {5000};     // 5 ms (200 Hz)
 TimeValue FApplication::time_last_event {};
 
+// FEvent friend function forward declaration
+void setSend (FEvent&, bool = true);
+void setQueued (FEvent&, bool = true);
+
 
 //----------------------------------------------------------------------
 // class FApplication
@@ -116,19 +121,19 @@ FApplication::~FApplication()  // destructor
 
 // public methods of FApplication
 //----------------------------------------------------------------------
-FApplication* FApplication::getApplicationObject()
+auto FApplication::getApplicationObject() -> FApplication*
 {
   return internal::var::app_object;
 }
 
 //----------------------------------------------------------------------
-FWidget* FApplication::getKeyboardWidget()
+auto FApplication::getKeyboardWidget() -> FWidget*
 {
   return keyboard_widget;
 }
 
 //----------------------------------------------------------------------
-FApplication::FLogPtr& FApplication::getLog()
+auto FApplication::getLog() -> FLogPtr&
 {
   // Global logger object
   static auto logger_ptr = new FLogPtr();
@@ -156,13 +161,21 @@ void FApplication::setLog (const FLogPtr& log)
 }
 
 //----------------------------------------------------------------------
-bool FApplication::isQuit()
+auto FApplication::isQuit() -> bool
 {
   return internal::var::app_object ? quit_now : true;
 }
 
 //----------------------------------------------------------------------
-int FApplication::exec()  // run
+#if defined(UNIT_TEST)
+void FApplication::start()  // Simulates application start
+{
+  quit_now = false;
+}
+#endif  // defined(UNIT_TEST)
+
+//----------------------------------------------------------------------
+auto FApplication::exec() -> int  // run
 {
   if ( quit_now )
   {
@@ -176,7 +189,7 @@ int FApplication::exec()  // run
 }
 
 //----------------------------------------------------------------------
-int FApplication::enterLoop()  // event loop
+auto FApplication::enterLoop() -> int  // event loop
 {
   loop_level++;
   quit_now = false;
@@ -212,17 +225,17 @@ void FApplication::quit() const
 }
 
 //----------------------------------------------------------------------
-bool FApplication::sendEvent (FObject* receiver, FEvent* event )
+auto FApplication::sendEvent (FObject* receiver, FEvent* event ) -> bool
 {
-  if ( quit_now || internal::var::exit_loop || ! (bool(receiver) && bool(event)) )
-    return false;
-
-  if ( ! isEventProcessable (receiver, event) )
+  if ( quit_now
+    || internal::var::exit_loop
+    || ! (bool(receiver) && bool(event))
+    || ! isEventProcessable (receiver, event) )
     return false;
 
   // Sends the event event directly to receiver
-  bool ret = receiver->event(event);
-  event->send = true;
+  const auto& ret = receiver->event(event);
+  setSend(*event);
   return ret;
 }
 
@@ -233,7 +246,7 @@ void FApplication::queueEvent (FObject* receiver, FEvent* event)
     return;
 
   // queue this event
-  event->queued = true;
+  setQueued(*event);
   event_queue.emplace_back (receiver, event);
 }
 
@@ -242,15 +255,15 @@ void FApplication::sendQueuedEvents()
 {
   while ( eventInQueue() )
   {
-    const EventPair& event_pair = event_queue.front();
-    event_pair.second->queued = false;
+    const auto& event_pair = event_queue.front();
+    setQueued(*event_pair.second, false);
     sendEvent(event_pair.first, event_pair.second);
     event_queue.pop_front();
   }
 }
 
 //----------------------------------------------------------------------
-bool FApplication::eventInQueue() const
+auto FApplication::eventInQueue() const -> bool
 {
   if ( internal::var::app_object )
     return ( ! event_queue.empty() );
@@ -259,7 +272,7 @@ bool FApplication::eventInQueue() const
 }
 
 //----------------------------------------------------------------------
-bool FApplication::removeQueuedEvent (const FObject* receiver)
+auto FApplication::removeQueuedEvent (const FObject* receiver) -> bool
 {
   if ( ! eventInQueue() )
     return false;
@@ -284,6 +297,12 @@ bool FApplication::removeQueuedEvent (const FObject* receiver)
   return retval;
 }
 
+//----------------------------------------------------------------------
+void FApplication::registerMouseHandler (const FMouseHandler& fn)
+{
+  mouse_handler_list.push_back(fn);
+}
+
 //----------------------------------------------------------------------
 void FApplication::initTerminal()
 {
@@ -335,7 +354,7 @@ void FApplication::setLogFile (const FString& filename)
   if ( log_stream.is_open() )
   {
     // Get the global logger object
-    FLog& log = *FApplication::getLog();
+    auto& log = *FApplication::getLog();
     log.setOutputStream(log_stream);
     log.enableTimestamp();
     log.setLineEnding (FLog::LineEnding::LF);
@@ -356,8 +375,8 @@ void FApplication::setKeyboardWidget (FWidget* widget)
 //----------------------------------------------------------------------
 void FApplication::closeConfirmationDialog (FWidget* w, FCloseEvent* ev)
 {
-  internal::var::app_object->unsetMoveSizeMode();
-  const FMessageBox::ButtonType ret = \
+  FApplication::unsetMoveResizeMode(FMouseData{});
+  const auto& ret = \
       FMessageBox::info ( w, "Quit"
                         , "Do you really want\n"
                           "to quit the program ?"
@@ -371,8 +390,7 @@ void FApplication::closeConfirmationDialog (FWidget* w, FCloseEvent* ev)
     ev->ignore();
 
     // Status bar restore after closing the FMessageBox
-    if ( getStatusBar() )
-      getStatusBar()->drawMessage();
+    drawStatusBarMessage();
   }
 }
 
@@ -414,7 +432,7 @@ void FApplication::init()
 
   // Initialize mouse control
   static auto& mouse = FMouseControl::getInstance();
-  auto cmd = [this] (const FMouseData& md) { this->mouseEvent(md); };
+  auto cmd = [this] (const auto& md) { this->mouseEvent(md); };
   FMouseCommand mouse_cmd (cmd);
   mouse.setEventCommand (mouse_cmd);
   // Set stdin number for a gpm-mouse
@@ -422,6 +440,10 @@ void FApplication::init()
   // Set the default double click interval
   mouse.setDblclickInterval (dblclick_interval);
 
+  // Restister mouse handler callbacks
+  registerMouseHandler (FApplication::determineClickedWidget);
+  registerMouseHandler (FApplication::unsetMoveResizeMode);
+
   // Initialize logging
   if ( ! getStartOptions().logfile_stream.is_open() )
     getLog()->setLineEnding(FLog::LineEnding::CRLF);
@@ -430,7 +452,7 @@ void FApplication::init()
 //----------------------------------------------------------------------
 void FApplication::setTerminalEncoding (const FString& enc_str)
 {
-  const FString& enc = enc_str.toLower();
+  const auto& enc = enc_str.toLower();
 
   if ( enc.includes("utf8") )
     getStartOptions().encoding = Encoding::UTF8;
@@ -452,9 +474,9 @@ void FApplication::setTerminalEncoding (const FString& enc_str)
 }
 
 //----------------------------------------------------------------------
-inline void FApplication::setLongOptions (std::vector<CmdOption>& long_options)
+inline auto FApplication::getLongOptions() -> const std::vector<CmdOption>&
 {
-  long_options =
+  static const std::vector<CmdOption>& long_options =
   {
     {"encoding",                 required_argument, nullptr,  'e' },
     {"log-file",                 required_argument, nullptr,  'l' },
@@ -462,6 +484,7 @@ inline void FApplication::setLongOptions (std::vector<CmdOption>& long_options)
     {"no-optimized-cursor",      no_argument,       nullptr,  'o' },
     {"no-terminal-detection",    no_argument,       nullptr,  'd' },
     {"no-terminal-data-request", no_argument,       nullptr,  'r' },
+    {"no-terminal-focus-events", no_argument,       nullptr,  'f' },
     {"no-color-change",          no_argument,       nullptr,  'c' },
     {"no-sgr-optimizer",         no_argument,       nullptr,  's' },
     {"vgafont",                  no_argument,       nullptr,  'v' },
@@ -477,45 +500,48 @@ inline void FApplication::setLongOptions (std::vector<CmdOption>& long_options)
 
     {nullptr,                    0,                 nullptr,  0   }
   };
+  return long_options;
 }
 
 //----------------------------------------------------------------------
 inline void FApplication::setCmdOptionsMap (CmdMap& cmd_map)
 {
-  auto enc = [] (const FString& s) { FApplication::setTerminalEncoding(s); };
-  auto log = [] (const FString& s) { FApplication::setLogFile(s); };
+  auto enc = [] (const auto& s) { FApplication::setTerminalEncoding(s); };
+  auto log = [] (const auto& s) { FApplication::setLogFile(s); };
   auto opt = &FApplication::getStartOptions;
 
   // --encoding
-  cmd_map['e'] = [enc] (const char* arg) { enc(FString(arg)); };
+  cmd_map['e'] = [enc] (const auto& arg) { enc(FString(arg)); };
   // --log-file
-  cmd_map['l'] = [log] (const char* arg) { log(FString(arg)); };
+  cmd_map['l'] = [log] (const auto& arg) { log(FString(arg)); };
   // --no-mouse
-  cmd_map['m'] = [opt] (const char*) { opt().mouse_support = false; };
+  cmd_map['m'] = [opt] (const auto&) { opt().mouse_support = false; };
   // --no-optimized-cursor
-  cmd_map['o'] = [opt] (const char*) { opt().cursor_optimisation = false; };
+  cmd_map['o'] = [opt] (const auto&) { opt().cursor_optimisation = false; };
   // --no-terminal-detection
-  cmd_map['d'] = [opt] (const char*) { opt().terminal_detection = false; };
+  cmd_map['d'] = [opt] (const auto&) { opt().terminal_detection = false; };
   // --no-terminal-data-request
-  cmd_map['r'] = [opt] (const char*) { opt().terminal_data_request = false; };
+  cmd_map['r'] = [opt] (const auto&) { opt().terminal_data_request = false; };
+  // --no-terminal-focus-events
+  cmd_map['f'] = [opt] (const auto&) { opt().terminal_focus_events = false; };
   // --no-color-change
-  cmd_map['c'] = [opt] (const char*) { opt().color_change = false; };
+  cmd_map['c'] = [opt] (const auto&) { opt().color_change = false; };
   // --no-sgr-optimizer
-  cmd_map['s'] = [opt] (const char*) { opt().sgr_optimizer = false; };
+  cmd_map['s'] = [opt] (const auto&) { opt().sgr_optimizer = false; };
   // --vgafont
-  cmd_map['v'] = [opt] (const char*) { opt().vgafont = true; };
+  cmd_map['v'] = [opt] (const auto&) { opt().vgafont = true; };
   // --newfont
-  cmd_map['n'] = [opt] (const char*) { opt().newfont = true; };
+  cmd_map['n'] = [opt] (const auto&) { opt().newfont = true; };
   // --dark-theme
-  cmd_map['t'] = [opt] (const char*) { opt().dark_theme = true; };
+  cmd_map['t'] = [opt] (const auto&) { opt().dark_theme = true; };
 #if defined(__FreeBSD__) || defined(__DragonFly__)
   // --no-esc-for-alt-meta
-  cmd_map['E'] = [opt] (const char*) { opt().meta_sends_escape = false; };
+  cmd_map['E'] = [opt] (const auto&) { opt().meta_sends_escape = false; };
   // --no-cursorstyle-change
-  cmd_map['C'] = [opt] (const char*) { opt().change_cursorstyle = false; };
+  cmd_map['C'] = [opt] (const auto&) { opt().change_cursorstyle = false; };
 #elif defined(__NetBSD__) || defined(__OpenBSD__)
   // --no-esc-for-alt-meta
-  cmd_map['E'] = [opt] (const char*) { opt().meta_sends_escape = false; };
+  cmd_map['E'] = [opt] (const auto&) { opt().meta_sends_escape = false; };
 #endif
 }
 
@@ -531,7 +557,7 @@ void FApplication::cmdOptions (const Args& args)
   std::transform ( args.cbegin()
                  , args.cend()
                  , argv.begin()
-                 , [] (const std::string& str)
+                 , [] (const auto& str)
                    {
                      return str.data();
                    }
@@ -541,26 +567,25 @@ void FApplication::cmdOptions (const Args& args)
   {
     opterr = 0;
     int idx{0};
-    std::vector<CmdOption> long_options{};
-    setLongOptions(long_options);
-    auto p = reinterpret_cast<const struct option*>(long_options.data());
+    const auto& long_options = getLongOptions();
+    const struct option* p = long_options.data();
     auto argv_data = const_cast<char* const*>(argv.data());
     const int opt = getopt_long (int(argc), argv_data, "", p, &idx);
 
     if ( opt == -1 )
       break;
 
-    const auto& entry = cmd_map[opt];
+    const auto& iter = cmd_map.find(opt);
 
-    if ( entry )
-      entry(optarg);
+    if ( iter != cmd_map.end() )
+      iter->second(optarg);
   }
 
   cmd_map.clear();
 }
 
 //----------------------------------------------------------------------
-inline FStartOptions& FApplication::getStartOptions()
+inline auto FApplication::getStartOptions() -> FStartOptions&
 {
   static auto& start_options = FStartOptions::getInstance();
   return start_options;
@@ -589,6 +614,8 @@ void FApplication::showParameterUsage()
     << "    Disable terminal detection\n"
     << "  --no-terminal-data-request"
     << "    Do not determine terminal font and title\n"
+    << "  --no-terminal-focus-events"
+    << "    Do not send focus-in and focus-out events\n"
     << "  --no-color-change         "
     << "    Do not redefine the color palette\n"
     << "  --no-sgr-optimizer        "
@@ -620,12 +647,13 @@ void FApplication::showParameterUsage()
 //----------------------------------------------------------------------
 inline void FApplication::destroyLog()
 {
+  // Reset the rdbuf of clog
+  std::clog << std::flush;
+  std::clog.rdbuf(default_clog_rdbuf);
+
   // Delete the logger
   const FLogPtr* logger = &(getLog());
   delete logger;
-
-  // Reset the rdbuf of clog
-  std::clog.rdbuf(default_clog_rdbuf);
 }
 
 //----------------------------------------------------------------------
@@ -635,20 +663,17 @@ inline void FApplication::findKeyboardWidget() const
 
   FWidget* widget{nullptr};
   auto focus = getFocusWidget();
-  auto move_size = getMoveSizeWidget();
+  auto move_size = getMoveResizeWidget();
 
   if ( focus )
   {
-    if ( move_size )
-      widget = move_size;
-    else
-      widget = focus;
+    widget = move_size ? move_size : focus;
   }
   else
   {
     widget = getMainWidget();
 
-    if ( widget && widget->numOfChildren() >= 1 )
+    if ( widget && widget->numOfFocusableChildren() >= 1 )
       widget->focusFirstChild();
   }
 
@@ -656,15 +681,16 @@ inline void FApplication::findKeyboardWidget() const
 }
 
 //----------------------------------------------------------------------
-inline bool FApplication::isKeyPressed() const
+inline auto FApplication::isKeyPressed(uInt64 ms) const -> bool
 {
   static auto& mouse = FMouseControl::getInstance();
   static auto& keyboard = FKeyboard::getInstance();
+  const auto blocking_time = (ms != 0U) ? ms : keyboard.getReadBlockingTime();
 
   if ( mouse.isGpmMouseEnabled() )
     return mouse.getGpmKeyPressed(keyboard.hasUnprocessedInput());
 
-  return (keyboard.isKeyPressed() || keyboard.hasPendingInput());
+  return (keyboard.isKeyPressed(blocking_time) || keyboard.hasPendingInput());
 }
 
 //----------------------------------------------------------------------
@@ -700,6 +726,11 @@ inline void FApplication::performKeyboardAction()
   {
     redraw();
   }
+  else if ( keyboard.getKey() == FKey::Term_Focus_In
+         || keyboard.getKey() == FKey::Term_Focus_Out )
+  {
+    processTerminalFocus (keyboard.getKey());  // Term focus-in/focus-out
+  }
   else
   {
     const bool acceptKeyDown = sendKeyDownEvent (keyboard_widget);
@@ -731,21 +762,15 @@ inline void FApplication::performMouseAction() const
     mouse.setRawData (FMouse::MouseType::Urxvt, buffer);
   }
 
-  keyboard.hasUnprocessedInput() = mouse.hasUnprocessedInput();
   queuingMouseInput();
 }
 
 //----------------------------------------------------------------------
-void FApplication::mouseEvent (const FMouseData& md)
+void FApplication::mouseEvent (const FMouseData& md) const
 {
-  determineClickedWidget (md);
+  for (const auto& mouse_handler : mouse_handler_list)
+    mouse_handler(md);  // Execute mouse handler
 
-  if ( ! FWidget::getClickedWidget() )
-    return;
-
-  unsetMoveSizeMode();
-  closeDropDown (md);
-  unselectMenubarItems (md);
   sendMouseEvent (md);
 }
 
@@ -758,7 +783,7 @@ inline void FApplication::sendEscapeKeyPressEvent() const
 }
 
 //----------------------------------------------------------------------
-inline bool FApplication::sendKeyDownEvent (FWidget* widget) const
+inline auto FApplication::sendKeyDownEvent (FWidget* widget) const -> bool
 {
   // Send key down event
   static const auto& keyboard = FKeyboard::getInstance();
@@ -768,7 +793,7 @@ inline bool FApplication::sendKeyDownEvent (FWidget* widget) const
 }
 
 //----------------------------------------------------------------------
-inline bool FApplication::sendKeyPressEvent (FWidget* widget) const
+inline auto FApplication::sendKeyPressEvent (FWidget* widget) const -> bool
 {
   // Send key press event
   static const auto& keyboard = FKeyboard::getInstance();
@@ -778,7 +803,7 @@ inline bool FApplication::sendKeyPressEvent (FWidget* widget) const
 }
 
 //----------------------------------------------------------------------
-inline bool FApplication::sendKeyUpEvent (FWidget* widget) const
+inline auto FApplication::sendKeyUpEvent (FWidget* widget) const -> bool
 {
   // Send key up event
   static const auto& keyboard = FKeyboard::getInstance();
@@ -816,7 +841,7 @@ inline void FApplication::sendKeyboardAccelerator()
 }
 
 //----------------------------------------------------------------------
-inline bool FApplication::hasDataInQueue() const
+inline auto FApplication::hasDataInQueue() const -> bool
 {
   static const auto& keyboard = FKeyboard::getInstance();
   static const auto& mouse = FMouseControl::getInstance();
@@ -843,10 +868,9 @@ void FApplication::queuingMouseInput() const
   if ( ! mouse.hasData() )
     return;
 
-  static auto& keyboard = FKeyboard::getInstance();
+  static const auto& keyboard = FKeyboard::getInstance();
   auto time_keypressed = keyboard.getKeyPressedTime();
   mouse.processEvent (time_keypressed);
-  keyboard.hasUnprocessedInput() = mouse.hasUnprocessedInput();
   mouse.clearEvent();
 }
 
@@ -872,12 +896,18 @@ inline void FApplication::processInput() const
 
   queuingKeyboardInput();
   queuingMouseInput();
-  processKeyboardEvent();
-  processMouseEvent();
+
+  do
+  {
+    processKeyboardEvent();
+    processMouseEvent();
+  }
+  while ( ! quit_now && hasDataInQueue() );
+  // quit_now is checked again to avoid an infinite loop on exit
 }
 
 //----------------------------------------------------------------------
-bool FApplication::processDialogSwitchAccelerator() const
+auto FApplication::processDialogSwitchAccelerator() const -> bool
 {
   static const auto& keyboard = FKeyboard::getInstance();
 
@@ -890,8 +920,8 @@ bool FApplication::processDialogSwitchAccelerator() const
 
     if ( s > 0 && s >= n )
     {
-      // unset the move/size mode
-      auto move_size = getMoveSizeWidget();
+      // unset the move/resize mode
+      auto move_size = getMoveResizeWidget();
 
       if ( move_size )
       {
@@ -910,7 +940,7 @@ bool FApplication::processDialogSwitchAccelerator() const
 }
 
 //----------------------------------------------------------------------
-bool FApplication::processAccelerator (const FWidget& widget) const
+auto FApplication::processAccelerator (const FWidget& widget) const -> bool
 {
   if ( widget.getAcceleratorList().empty() )
     return false;
@@ -922,7 +952,7 @@ bool FApplication::processAccelerator (const FWidget& widget) const
     if ( item.key == keyboard.getKey() )
     {
       // unset the move/size mode
-      auto move_size = getMoveSizeWidget();
+      auto move_size = getMoveResizeWidget();
 
       if ( move_size )
       {
@@ -942,6 +972,31 @@ bool FApplication::processAccelerator (const FWidget& widget) const
   return false;
 }
 
+//----------------------------------------------------------------------
+void FApplication::processTerminalFocus (const FKey& key)
+{
+  auto root_widget = getRootWidget();
+
+  if ( ! root_widget
+    || ( key != FKey::Term_Focus_In && key != FKey::Term_Focus_Out ) )
+    return;
+
+  // unset the move/size mode
+  auto move_size = getMoveResizeWidget();
+
+  if ( move_size )
+  {
+    setMoveSizeWidget(nullptr);
+    move_size->redraw();
+  }
+
+  auto event = ( key == FKey::Term_Focus_In )
+              ? Event::TerminalFocusIn
+              : Event::TerminalFocusOut;
+  FFocusEvent tf_ev (event);
+  sendEvent (root_widget, &tf_ev);
+}
+
 //----------------------------------------------------------------------
 void FApplication::determineClickedWidget (const FMouseData& md)
 {
@@ -963,78 +1018,36 @@ void FApplication::determineClickedWidget (const FMouseData& md)
   // Determine the window object on the current click position
   auto window = FWindow::getWindowWidgetAt (mouse_position);
 
-  if ( window )
-  {
-    // Determine the widget at the current click position
-    auto child = window->childWidgetAt(mouse_position);
-    clicked_widget = ( child != nullptr ) ? child : window;
-    setClickedWidget (clicked_widget);
-  }
+  if ( ! window )
+    return;
+
+  // Determine the widget at the current click position
+  auto child = window->childWidgetAt(mouse_position);
+  clicked_widget = ( child != nullptr ) ? child : window;
+  setClickedWidget (clicked_widget);
 }
 
 //----------------------------------------------------------------------
-void FApplication::unsetMoveSizeMode() const
+void FApplication::unsetMoveResizeMode (const FMouseData&)
 {
-  // Unset the move/size mode
+  // Unset the move or resize mode
 
-  auto& move_size = getMoveSizeWidget();
+  auto& move_size = getMoveResizeWidget();
 
-  if ( move_size )
-  {
-    FWidget* w{nullptr};
-    std::swap(w, move_size);  // Clear move_size_widget
-    w->redraw();
-  }
-}
-
-//----------------------------------------------------------------------
-void FApplication::closeDropDown (const FMouseData& md) const
-{
-  // Close the open menu
-
-  if ( md.isMoved() )
+  if ( ! move_size )
     return;
 
-  const auto& mouse_position = md.getPos();
-  finalcut::closeDropDown (this, mouse_position);
-}
-
-//----------------------------------------------------------------------
-void FApplication::unselectMenubarItems (const FMouseData& md) const
-{
-  // Unselect the menu bar items
-
-  const auto& openmenu = FWidget::getOpenMenu();
-  auto menu_bar = FWidget::getMenuBar();
-
-  if ( openmenu || md.isMoved() )
-    return;
-
-  if ( ! (menu_bar && menu_bar->hasSelectedItem()) )
-    return;
-
-  const auto& mouse_position = md.getPos();
-
-  if ( ! menu_bar->getTermGeometry().contains(mouse_position) )
-  {
-    if ( FWidget::getStatusBar() )
-      FWidget::getStatusBar()->clearMessage();
-
-    menu_bar->resetMenu();
-    menu_bar->redraw();
-
-    // No widget was been clicked
-    if ( ! FWidget::getClickedWidget() )
-      FWindow::switchToPrevWindow(this);
-
-    if ( FWidget::getStatusBar() )
-      FWidget::getStatusBar()->drawMessage();
-  }
+  FWidget* w{nullptr};
+  std::swap(w, move_size);  // Clear move_resize_widget
+  w->redraw();
 }
 
 //----------------------------------------------------------------------
 void FApplication::sendMouseEvent (const FMouseData& md) const
 {
+  if ( ! FWidget::getClickedWidget() )  // No widget was clicked
+    return;
+
   const auto& mouse_position = md.getPos();
   MouseButton key_state{MouseButton::None};
 
@@ -1226,7 +1239,7 @@ void FApplication::sendWheelEvent ( const FMouseData& md
 }
 
 //----------------------------------------------------------------------
-FWidget* FApplication::processParameters (const Args& args)
+auto FApplication::processParameters (const Args& args) -> FWidget*
 {
   if ( args.size() > 1 && (args[1] == "--help" || args[1] == "-h") )
   {
@@ -1241,11 +1254,10 @@ FWidget* FApplication::processParameters (const Args& args)
 //----------------------------------------------------------------------
 void FApplication::processResizeEvent()
 {
-  auto foutput_ptr = FVTerm::getFOutput();
-
   if ( ! has_terminal_resized )  // A SIGWINCH signal was received
     return;
 
+  auto foutput_ptr = FVTerm::getFOutput();
   foutput_ptr->detectTerminalSize();  // Detect and save the current terminal size
   static auto& mouse = FMouseControl::getInstance();
   mouse.setMaxWidth (uInt16(getDesktopWidth()));
@@ -1258,6 +1270,13 @@ void FApplication::processResizeEvent()
     foutput_ptr->commitTerminalResize();
 }
 
+//----------------------------------------------------------------------
+void FApplication::processDialogResizeMove() const
+{
+  for (auto&& dialog : *FWidget::getDialogList())
+    dialog->flushChanges();
+}
+
 //----------------------------------------------------------------------
 void FApplication::processCloseWidget()
 {
@@ -1291,22 +1310,27 @@ void FApplication::processLogger() const
 }
 
 //----------------------------------------------------------------------
-bool FApplication::processNextEvent()
+auto FApplication::processNextEvent() -> bool
 {
   uInt num_events{0};
 
   if ( hasDataInQueue() || hasTerminalResized() || isNextEventTimeout() )
   {
-    time_last_event = FObject::getCurrentTime();
+    time_last_event = FObjectTimer::getCurrentTime();
     num_events += processTimerEvent();
     processInput();
     processResizeEvent();
     processCloseWidget();
     sendQueuedEvents();
+    processDialogResizeMove();
     processTerminalUpdate();  // after terminal changes
     flush();
     processLogger();
   }
+  else if ( isKeyPressed(next_event_wait) )
+  {
+    time_last_event = TimeValue{};
+  }
 
   processExternalUserEvent();
   return ( num_events > 0 );
@@ -1319,7 +1343,7 @@ void FApplication::performTimerAction (FObject* receiver, FEvent* event)
 }
 
 //----------------------------------------------------------------------
-inline bool FApplication::hasTerminalResized()
+inline auto FApplication::hasTerminalResized() -> bool
 {
   auto foutput_ptr = FVTerm::getFOutput();
   has_terminal_resized = foutput_ptr->hasTerminalResized();
@@ -1327,8 +1351,8 @@ inline bool FApplication::hasTerminalResized()
 }
 
 //----------------------------------------------------------------------
-bool FApplication::isEventProcessable ( FObject* receiver
-                                      , const FEvent* event )
+auto FApplication::isEventProcessable ( FObject* receiver
+                                      , const FEvent* event ) -> bool
 {
   if ( ! receiver->isWidget() )  // No restrictions for non-widgets
     return true;
@@ -1337,16 +1361,13 @@ bool FApplication::isEventProcessable ( FObject* receiver
 
   if ( getModalDialogCounter() > 0 )
   {
-    const FWidget* window;
-
-    if ( widget->isWindowWidget() )
-      window = widget;
-    else
-      window = FWindow::getWindowWidget(widget);
+    const FWidget* window = widget->isWindowWidget()
+                          ? widget
+                          : FWindow::getWindowWidget(widget);
 
     // block events for widgets in non modal windows
     if ( window
-      && ! window->getFlags().modal
+      && ! window->getFlags().visibility.modal
       && ! window->isMenuWidget() )
     {
       constexpr std::array<const Event, 13> blocked_events
@@ -1368,7 +1389,7 @@ bool FApplication::isEventProcessable ( FObject* receiver
 
       if ( std::any_of( blocked_events.cbegin()
                       , blocked_events.cend()
-                      , [&event](const Event& ev)
+                      , [&event](const auto& ev)
                         {
                           return ev == event->getType();
                         } ) )
@@ -1385,17 +1406,46 @@ bool FApplication::isEventProcessable ( FObject* receiver
 }
 
 //----------------------------------------------------------------------
-bool FApplication::isNextEventTimeout()
+auto FApplication::isNextEventTimeout() -> bool
 {
-  return FObject::isTimeout(time_last_event, next_event_wait);
+  return FObjectTimer::isTimeout(time_last_event, next_event_wait);
+}
+
+
+// Friend functions definition of FEvent
+//----------------------------------------------------------------------
+void setSend (FEvent& event, bool state)
+{
+  event.send = state;
+}
+
+//----------------------------------------------------------------------
+void setQueued (FEvent& event, bool state)
+{
+  event.queued = state;
 }
 
 
 // FLog non-member operators
 //----------------------------------------------------------------------
-std::ostream& operator << (std::ostream& outstr, FLog::LogLevel l)
+auto operator << (std::ostream& outstr, FLog::LogLevel l) -> std::ostream&
 {
-  *FApplication::getLog() << l;
+  try
+  {
+    *FApplication::getLog() << l;
+  }
+  catch (const std::invalid_argument&)
+  {
+    try
+    {
+      *FApplication::getLog() << FLog::LogLevel::Info;
+    }
+    catch (const std::invalid_argument&)  // Avoid being thrown again
+    {
+      *FApplication::getLog();
+    }
+  }
+
   return outstr;
 }
 
