@@ -9,7 +9,7 @@ namespace finalcut
 namespace fc
 {
 
-constexpr unsigned char __9x16graph[] =
+constexpr unsigned char F_9x16graph[] =
 {
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  /* 0 */
   0x00, 0x00, 0x7e, 0x81, 0xa5, 0x81, 0x81, 0xbd, 0x99, 0x81, 0x81, 0x7e, 0x00, 0x00, 0x00, 0x00,  /* 1 */
