@@ -3,7 +3,7 @@
 *                                                                      *
 * This file is part of the FINAL CUT widget toolkit                    *
 *                                                                      *
-* Copyright 2018-2021 Markus Gans                                      *
+* Copyright 2018-2023 Markus Gans                                      *
 *                                                                      *
 * FINAL CUT is free software; you can redistribute it and/or modify    *
 * it under the terms of the GNU Lesser General Public License as       *
@@ -20,6 +20,10 @@
 * <http://www.gnu.org/licenses/>.                                      *
 ***********************************************************************/
 
+#if defined(__CYGWIN__)
+  #include "final/fconfig.h"  // includes _GNU_SOURCE for fd_set
+#endif
+
 #include <cstdio>
 #include <cstring>
 #include <unistd.h>
@@ -41,8 +45,28 @@
 namespace finalcut
 {
 
-// static class attributes
-FMouseData* FMouseControl::current_mouse_event{nullptr};
+// Function prototypes
+template <typename NumT, typename UnaryPredicate>
+auto parseNumberIf (const char*&, NumT&, UnaryPredicate) -> bool;
+
+// non-member functions
+//----------------------------------------------------------------------
+template <typename NumT, typename UnaryPredicate>
+inline auto parseNumberIf ( const char*& p
+                          , NumT& number
+                          , UnaryPredicate up ) -> bool
+{
+  while ( *p && up(*p) )
+  {
+    if ( ! std::isdigit(*p) )
+      return false;
+
+    number = 10 * number + (*p - '0');
+    p++;
+  }
+
+  return true;
+}
 
 //----------------------------------------------------------------------
 // class FMouseData
@@ -55,90 +79,97 @@ FMouseData::~FMouseData() noexcept = default;  // destructor
 
 // public methods of FMouseData
 //----------------------------------------------------------------------
-FString FMouseData::getClassName() const
+auto FMouseData::getClassName() const -> FString
 {
   return "FMouseData";
 }
 
 //----------------------------------------------------------------------
-const FPoint& FMouseData::getPos() const & noexcept
+auto FMouseControl::getCurrentMouseEvent() -> FMouseDataPtr&
+{
+  static const auto& current_mouse_event = std::make_unique<FMouseDataPtr>();
+  return *current_mouse_event;
+}
+
+//----------------------------------------------------------------------
+auto FMouseData::getPos() const & noexcept -> const FPoint&
 {
   return mouse;
 }
 
 //----------------------------------------------------------------------
-bool FMouseData::isLeftButtonPressed() const noexcept
+auto FMouseData::isLeftButtonPressed() const noexcept -> bool
 {
   return getButtonState().left_button == State::Pressed;
 }
 
 //----------------------------------------------------------------------
-bool FMouseData::isLeftButtonReleased() const noexcept
+auto FMouseData::isLeftButtonReleased() const noexcept -> bool
 {
   return getButtonState().left_button == State::Released;
 }
 
 //----------------------------------------------------------------------
-bool FMouseData::isLeftButtonDoubleClick() const noexcept
+auto FMouseData::isLeftButtonDoubleClick() const noexcept -> bool
 {
   return getButtonState().left_button == State::DoubleClick;
 }
 
 //----------------------------------------------------------------------
-bool FMouseData::isRightButtonPressed() const noexcept
+auto FMouseData::isRightButtonPressed() const noexcept -> bool
 {
   return getButtonState().right_button == State::Pressed;
 }
 
 //----------------------------------------------------------------------
-bool FMouseData::isRightButtonReleased() const noexcept
+auto FMouseData::isRightButtonReleased() const noexcept -> bool
 {
   return getButtonState().right_button == State::Released;
 }
 //----------------------------------------------------------------------
-bool FMouseData::isMiddleButtonPressed() const noexcept
+auto FMouseData::isMiddleButtonPressed() const noexcept -> bool
 {
   return getButtonState().middle_button == State::Pressed;
 }
 
 //----------------------------------------------------------------------
-bool FMouseData::isMiddleButtonReleased() const noexcept
+auto FMouseData::isMiddleButtonReleased() const noexcept -> bool
 {
   return getButtonState().middle_button == State::Released;
 }
 
 //----------------------------------------------------------------------
-bool FMouseData::isShiftKeyPressed() const noexcept
+auto FMouseData::isShiftKeyPressed() const noexcept -> bool
 {
   return getButtonState().shift_button;
 }
 
 //----------------------------------------------------------------------
-bool FMouseData::isControlKeyPressed() const noexcept
+auto FMouseData::isControlKeyPressed() const noexcept -> bool
 {
   return getButtonState().control_button;
 }
 
 //----------------------------------------------------------------------
-bool FMouseData::isMetaKeyPressed() const noexcept
+auto FMouseData::isMetaKeyPressed() const noexcept -> bool
 {
   return getButtonState().meta_button;
 }
 
 //----------------------------------------------------------------------
-bool FMouseData::isWheelUp() const noexcept
+auto FMouseData::isWheelUp() const noexcept -> bool
 {
   return getButtonState().wheel_up;
 }
 
 //----------------------------------------------------------------------
-bool FMouseData::isWheelDown() const noexcept
+auto FMouseData::isWheelDown() const noexcept -> bool
 {
   return getButtonState().wheel_down;
 }
 
 //----------------------------------------------------------------------
-bool FMouseData::isMoved() const noexcept
+auto FMouseData::isMoved() const noexcept -> bool
 {
   return getButtonState().mouse_moved;
 }
@@ -160,13 +191,13 @@ void FMouseData::clearButtonState() noexcept
 
 // protected methods of FMouseData
 //----------------------------------------------------------------------
-inline FMouseData::FMouseButton& FMouseData::getButtonState() & noexcept
+inline auto FMouseData::getButtonState() & noexcept -> FMouseButton&
 {
   return b_state;
 }
 
 //----------------------------------------------------------------------
-inline const FMouseData::FMouseButton& FMouseData::getButtonState() const & noexcept
+inline auto FMouseData::getButtonState() const & noexcept -> const FMouseButton&
 {
   return b_state;
 }
@@ -193,13 +224,13 @@ FMouse::FMouse()
 
 // public methods of FMouse
 //----------------------------------------------------------------------
-FString FMouse::getClassName() const
+auto FMouse::getClassName() const -> FString
 {
   return "FMouse";
 }
 
 //----------------------------------------------------------------------
-FMouse::MouseType FMouse::getMouseTypeID() const noexcept
+auto FMouse::getMouseTypeID() const noexcept -> MouseType
 {
   return MouseType_id;
 }
@@ -229,13 +260,13 @@ void FMouse::setDblclickInterval (const uInt64 timeout) noexcept
 }
 
 //----------------------------------------------------------------------
-bool FMouse::hasEvent() const noexcept
+auto FMouse::hasEvent() const noexcept -> bool
 {
   return mouse_event_occurred;
 }
 
 //----------------------------------------------------------------------
-bool FMouse::hasUnprocessedInput() const noexcept
+auto FMouse::hasUnprocessedInput() const noexcept -> bool
 {
   return unprocessed_buffer_data;
 }
@@ -243,31 +274,31 @@ bool FMouse::hasUnprocessedInput() const noexcept
 
 // protected methods of FMouse
 //----------------------------------------------------------------------
-const FPoint& FMouse::getNewPos() const & noexcept
+auto FMouse::getNewPos() const & noexcept -> const FPoint&
 {
   return new_mouse_position;
 }
 
 //----------------------------------------------------------------------
-uInt16 FMouse::getMaxWidth() const noexcept
+auto FMouse::getMaxWidth() const noexcept -> uInt16
 {
   return max_width;
 }
 
 //----------------------------------------------------------------------
-uInt16 FMouse::getMaxHeight() const noexcept
+auto FMouse::getMaxHeight() const noexcept -> uInt16
 {
   return max_height;
 }
 
 //----------------------------------------------------------------------
-uInt64 FMouse::getDblclickInterval() const noexcept
+auto FMouse::getDblclickInterval() const noexcept -> uInt64
 {
   return dblclick_interval;
 }
 
 //----------------------------------------------------------------------
-TimeValue FMouse::getMousePressedTime() const noexcept
+auto FMouse::getMousePressedTime() const noexcept -> TimeValue
 {
   return time_mousepressed;
 }
@@ -284,6 +315,12 @@ void FMouse::setNewPos (int x, int y) noexcept
   new_mouse_position.setPoint (x, y);
 }
 
+//----------------------------------------------------------------------
+void FMouse::useNewPos() noexcept
+{
+  setPos(new_mouse_position);
+}
+
 //----------------------------------------------------------------------
 void FMouse::setPending (bool is_pending) noexcept
 {
@@ -309,9 +346,9 @@ void FMouse::setEvent() noexcept
 }
 
 //----------------------------------------------------------------------
-bool FMouse::isDblclickTimeout (const TimeValue& time) const
+auto FMouse::isDblclickTimeout (const TimeValue& time) const -> bool
 {
-  return FObject::isTimeout (time, dblclick_interval);
+  return FObjectTimer::isTimeout (time, dblclick_interval);
 }
 
 
@@ -332,7 +369,7 @@ FMouseGPM::FMouseGPM()
 
 // public methods of FMouseX11
 //----------------------------------------------------------------------
-FString FMouseGPM::getClassName() const
+auto FMouseGPM::getClassName() const -> FString
 {
   return "FMouseGPM";
 }
@@ -344,7 +381,7 @@ void FMouseGPM::setStdinNo (int file_descriptor) noexcept
 }
 
 //----------------------------------------------------------------------
-bool FMouseGPM::hasData() noexcept
+auto FMouseGPM::hasData() noexcept -> bool
 {
   return has_gpm_mouse_data;
 }
@@ -414,7 +451,7 @@ void FMouseGPM::processEvent (const TimeValue&)
 }
 
 //----------------------------------------------------------------------
-bool FMouseGPM::gpmMouse (bool enable)
+auto FMouseGPM::gpmMouse (bool enable) -> bool
 {
   // activate/deactivate the gpm mouse support
 
@@ -443,7 +480,7 @@ bool FMouseGPM::gpmMouse (bool enable)
 }
 
 //----------------------------------------------------------------------
-bool FMouseGPM::hasSignificantEvents() const noexcept
+auto FMouseGPM::hasSignificantEvents() const noexcept -> bool
 {
   return ! (gpm_ev.type & GPM_MOVE)
       || gpm_ev.wdy != 0
@@ -499,7 +536,7 @@ void FMouseGPM::interpretKeyUp() noexcept
 }
 
 //----------------------------------------------------------------------
-bool FMouseGPM::getGpmKeyPressed (bool is_pending)
+auto FMouseGPM::getGpmKeyPressed (bool is_pending) -> bool
 {
   setPending(is_pending);
   has_gpm_mouse_data = false;
@@ -521,9 +558,9 @@ void FMouseGPM::drawPointer() const
 }
 
 //----------------------------------------------------------------------
-FMouseGPM::gpmEventType FMouseGPM::gpmEvent (bool clear) const
+auto FMouseGPM::gpmEvent (bool clear) const -> gpmEventType
 {
-  const int max = ( gpm_fd > stdin_no ) ? gpm_fd : stdin_no;
+  const int max = std::max(gpm_fd, stdin_no);
   fd_set ifds{};
   struct timeval tv{};
 
@@ -566,13 +603,13 @@ FMouseX11::FMouseX11()
 
 // public methods of FMouseX11
 //----------------------------------------------------------------------
-FString FMouseX11::getClassName() const
+auto FMouseX11::getClassName() const -> FString
 {
   return "FMouseX11";
 }
 
 //----------------------------------------------------------------------
-bool FMouseX11::hasData() noexcept
+auto FMouseX11::hasData() noexcept -> bool
 {
   return ( x11_mouse[0] );
 }
@@ -583,24 +620,12 @@ void FMouseX11::setRawData (FKeyboard::keybuffer& fifo_buf) noexcept
   // Import the X11 xterm mouse protocol (SGR-Mode) raw mouse data
 
   static constexpr std::size_t len = 6;
-  const std::size_t fifo_buf_size{sizeof(fifo_buf)};
-  std::size_t n{};
   x11_mouse[0] = fifo_buf[3];
   x11_mouse[1] = fifo_buf[4];
   x11_mouse[2] = fifo_buf[5];
   x11_mouse[3] = '\0';
-
-  // Remove founded entry
-  for (n = len; n < fifo_buf_size; n++)
-    fifo_buf[n - len] = fifo_buf[n];
-
-  n = fifo_buf_size - len;
-
-  // Fill rest with '\0'
-  for (; n < fifo_buf_size; n++)
-    fifo_buf[n] = '\0';
-
-  setPending(bool(fifo_buf[0] != '\0'));
+  fifo_buf.pop(len);  // Remove founded entry
+  setPending(fifo_buf.hasData());
 }
 
 //----------------------------------------------------------------------
@@ -611,7 +636,7 @@ void FMouseX11::processEvent (const TimeValue& time)
   const auto& mouse_position = getPos();
   const auto x = uChar(x11_mouse[1] - 0x20);
   const auto y = uChar(x11_mouse[2] - 0x20);
-  const int btn = x11_mouse[0];
+  const int btn = uChar(x11_mouse[0]);
   setNewPos (x, y);
   clearButtonState();
   setKeyState (btn);
@@ -667,24 +692,11 @@ void FMouseX11::setButtonState (const int btn, const TimeValue& time) noexcept
 {
   // Get the x11 mouse button state
 
-  const auto& mouse_position = getPos();
-
   switch ( btn )
   {
     case button1_pressed:
     case button1_pressed_move:
-      if ( mouse_position == getNewPos()
-        && x11_button_state == all_buttons_released
-        && ! isDblclickTimeout(getMousePressedTime()) )
-      {
-        resetMousePressedTime();
-        getButtonState().left_button = State::DoubleClick;
-      }
-      else
-      {
-        setMousePressedTime (time);  // save click time
-        getButtonState().left_button = State::Pressed;
-      }
+      handleButton1Pressed(time);
       break;
 
     case button2_pressed:
@@ -700,26 +712,7 @@ void FMouseX11::setButtonState (const int btn, const TimeValue& time) noexcept
       break;
 
     case all_buttons_released:
-      switch ( x11_button_state & button_mask )
-      {
-        case button1_pressed:
-        case button1_pressed_move:
-          getButtonState().left_button = State::Released;
-          break;
-
-        case button2_pressed:
-        case button2_pressed_move:
-          getButtonState().middle_button = State::Released;
-          break;
-
-        case button3_pressed:
-        case button3_pressed_move:
-          getButtonState().right_button = State::Released;
-          break;
-
-        default:
-          break;
-      }
+      handleButtonRelease();
       break;
 
     case button_up:
@@ -737,6 +730,48 @@ void FMouseX11::setButtonState (const int btn, const TimeValue& time) noexcept
   }
 }
 
+//----------------------------------------------------------------------
+void FMouseX11::handleButton1Pressed (const TimeValue& time) noexcept
+{
+  if ( getPos() == getNewPos()
+    && x11_button_state == all_buttons_released
+    && ! isDblclickTimeout(getMousePressedTime()) )
+  {
+    resetMousePressedTime();
+    getButtonState().left_button = State::DoubleClick;
+  }
+  else
+  {
+    setMousePressedTime (time);  // save click time
+    getButtonState().left_button = State::Pressed;
+  }
+}
+
+//----------------------------------------------------------------------
+void FMouseX11::handleButtonRelease() noexcept
+{
+  switch ( x11_button_state & button_mask )
+  {
+    case button1_pressed:
+    case button1_pressed_move:
+      getButtonState().left_button = State::Released;
+      break;
+
+    case button2_pressed:
+    case button2_pressed_move:
+      getButtonState().middle_button = State::Released;
+      break;
+
+    case button3_pressed:
+    case button3_pressed_move:
+      getButtonState().right_button = State::Released;
+      break;
+
+    default:
+      break;
+  }
+}
+
 
 //----------------------------------------------------------------------
 // class FMouseSGR
@@ -751,13 +786,13 @@ FMouseSGR::FMouseSGR()
 
 // public methods of FMouseSGR
 //----------------------------------------------------------------------
-FString FMouseSGR::getClassName() const
+auto FMouseSGR::getClassName() const -> FString
 {
   return "FMouseSGR";
 }
 
 //----------------------------------------------------------------------
-bool FMouseSGR::hasData() noexcept
+auto FMouseSGR::hasData() noexcept -> bool
 {
   return ( sgr_mouse[0] );
 }
@@ -767,81 +802,69 @@ void FMouseSGR::setRawData (FKeyboard::keybuffer& fifo_buf) noexcept
 {
   // Import the X11 xterm mouse protocol (SGR-Mode) raw mouse data
 
-  const std::size_t fifo_buf_size = sizeof(fifo_buf);
-  std::size_t len = stringLength(fifo_buf);
+  const auto max = fifo_buf.getSize();
+  std::size_t len{0};
   std::size_t n{3};
 
-  while ( n < len && n <= MOUSE_BUF_SIZE + 1 )
+  while ( n < max )
   {
     sgr_mouse[n - 3] = fifo_buf[n];
-    n++;
 
     if ( fifo_buf[n] == 'M' || fifo_buf[n] == 'm' )
-      len = n + 1;
+    {
+      n++;
+      len = n;
+      break;
+    }
+
+    n++;
   }
 
   sgr_mouse[n - 3] = '\0';
-
-  for (n = len; n < fifo_buf_size; n++)  // Remove founded entry
-    fifo_buf[n - len] = fifo_buf[n];
-
-  n = fifo_buf_size - len;
-
-  for (; n < fifo_buf_size; n++)       // Fill rest with '\0'
-    fifo_buf[n] = '\0';
-
-  setPending(bool(fifo_buf[0] != '\0'));
+  fifo_buf.pop(len);  // Remove founded entry
+  setPending(fifo_buf.hasData());
 }
 
 //----------------------------------------------------------------------
 void FMouseSGR::processEvent (const TimeValue& time)
 {
   const auto& mouse_position = getPos();
-  uInt16 x{0};
-  uInt16 y{0};
+  sInt16 x{0};
+  sInt16 y{0};
   int btn{0};
 
-  // parse the SGR mouse string
-  const char* p = sgr_mouse;
+  // Parse the SGR mouse string
+  const char* p = sgr_mouse.data();
 
-  while ( *p && *p != ';' )
+  // Parse button
+  if ( ! parseNumberIf (p, btn, [] (char ch) { return ch != ';'; }) )
   {
-    if ( *p < '0' || *p > '9')
-    {
-      clearEvent();
-      sgr_mouse[0] = '\0';  // Delete already interpreted data
-      return;
-    }
-
-    btn = 10 * btn + (*p - '0');
-    p++;
+    clearEvent();
+    sgr_mouse[0] = '\0';  // Delete already interpreted data
+    return;
   }
 
-  while ( *p++ && *p != ';' )
-  {
-    if ( *p < '0' || *p > '9')
-    {
-      clearEvent();
-      sgr_mouse[0] = '\0';  // Delete already interpreted data
-      return;
-    }
+  if ( *p ) p++;  // ship one character after the number
 
-    x = uInt16(10 * x + (*p - '0'));
+  // Parse x-value
+  if ( ! parseNumberIf (p, x, [] (char ch) { return ch != ';'; }) )
+  {
+    clearEvent();
+    sgr_mouse[0] = '\0';  // Delete already interpreted data
+    return;
   }
 
-  while ( *p++ && *p != 'M' && *p != 'm' )
-  {
-    if ( *p < '0' || *p > '9')
-    {
-      clearEvent();
-      sgr_mouse[0] = '\0';  // Delete already interpreted data
-      return;
-    }
+  if ( *p ) p++;  // ship one character after the number
 
-    y = uInt16(10 * y + (*p - '0'));
+  // Parse y-value
+  if ( ! parseNumberIf (p, y, [] (char ch) { return ch != 'M' && ch != 'm'; }) )
+  {
+    clearEvent();
+    sgr_mouse[0] = '\0';  // Delete already interpreted data
+    return;
   }
 
-  setNewPos (x, y);
+  setNewPos (x, y);  // Update the mouse state
   clearButtonState();
   setKeyState (btn);
   setMoveState (mouse_position, btn);
@@ -862,7 +885,7 @@ void FMouseSGR::processEvent (const TimeValue& time)
   }
 
   setEvent();
-  setPos (FPoint{x, y});
+  useNewPos();
   // Get the button state from string
   sgr_button_state = uChar(((*p & 0x20) << 2) + btn);
   // Delete already interpreted data
@@ -900,24 +923,11 @@ void FMouseSGR::setPressedButtonState ( const int btn
 {
   // Gets the extended x11 mouse mode (SGR) status for pressed buttons
 
-  const auto& mouse_position = getPos();
-
   switch ( btn )
   {
     case button1:
     case button1_move:
-      if ( mouse_position == getNewPos()
-        && (((sgr_button_state & 0x80) >> 2) + 'M') == released
-        && ! isDblclickTimeout(getMousePressedTime()) )
-      {
-        resetMousePressedTime();
-        getButtonState().left_button = State::DoubleClick;
-      }
-      else
-      {
-        setMousePressedTime (time);  // save click time
-        getButtonState().left_button = State::Pressed;
-      }
+      handleButton1Pressed(time);
       break;
 
     case button2:
@@ -947,6 +957,23 @@ void FMouseSGR::setPressedButtonState ( const int btn
   }
 }
 
+//----------------------------------------------------------------------
+void FMouseSGR::handleButton1Pressed (const TimeValue& time) noexcept
+{
+  if ( getPos() == getNewPos()
+    && (((sgr_button_state & 0x80) >> 2) + 'M') == released
+    && ! isDblclickTimeout(getMousePressedTime()) )
+  {
+    resetMousePressedTime();
+    getButtonState().left_button = State::DoubleClick;
+  }
+  else
+  {
+    setMousePressedTime (time);  // save click time
+    getButtonState().left_button = State::Pressed;
+  }
+}
+
 //----------------------------------------------------------------------
 void FMouseSGR::setReleasedButtonState (const int btn) noexcept
 {
@@ -988,13 +1015,13 @@ FMouseUrxvt::FMouseUrxvt()
 
 // public methods of FMouseUrxvt
 //----------------------------------------------------------------------
-FString FMouseUrxvt::getClassName() const
+auto FMouseUrxvt::getClassName() const -> FString
 {
   return "FMouseUrxvt";
 }
 
 //----------------------------------------------------------------------
-bool FMouseUrxvt::hasData() noexcept
+auto FMouseUrxvt::hasData() noexcept -> bool
 {
   return ( urxvt_mouse[0] );
 }
@@ -1004,30 +1031,27 @@ void FMouseUrxvt::setRawData (FKeyboard::keybuffer& fifo_buf) noexcept
 {
   // Import the X11 xterm mouse protocol (Urxvt-Mode) raw mouse data
 
-  const std::size_t fifo_buf_size = sizeof(fifo_buf);
-  std::size_t len = stringLength(fifo_buf);
+  const auto max = fifo_buf.getSize();
+  std::size_t len{0};
   std::size_t n{2};
 
-  while ( n < len && n <= MOUSE_BUF_SIZE )
+  while ( n < max )
   {
     urxvt_mouse[n - 2] = fifo_buf[n];
-    n++;
 
     if ( fifo_buf[n] == 'M' || fifo_buf[n] == 'm' )
-      len = n + 1;
+    {
+      n++;
+      len = n;
+      break;
+    }
+
+    n++;
   }
 
   urxvt_mouse[n - 2] = '\0';
-
-  for (n = len; n < fifo_buf_size; n++)  // Remove founded entry
-    fifo_buf[n - len] = fifo_buf[n];
-
-  n = fifo_buf_size - len;
-
-  for (; n < fifo_buf_size; n++)       // Fill rest with '\0'
-    fifo_buf[n] = '\0';
-
-  setPending(bool(fifo_buf[0] != '\0'));
+  fifo_buf.pop(len);  // Remove founded entry
+  setPending(fifo_buf.hasData());
 }
 
 //----------------------------------------------------------------------
@@ -1036,26 +1060,21 @@ void FMouseUrxvt::processEvent (const TimeValue& time)
   // Parse and interpret the X11 xterm mouse string (Urxvt-Mode)
 
   const auto& mouse_position = getPos();
-  uInt16 x{0};
-  uInt16 y{0};
+  sInt16 x{0};
+  sInt16 y{0};
   int btn{0};
 
   // Parse the Urxvt mouse string
-  const char* p = urxvt_mouse;
+  const char* p = urxvt_mouse.data();
   bool x_neg{false};
   bool y_neg{false};
 
-  while ( *p && *p != ';' )
+  // Parse button
+  if ( ! parseNumberIf (p, btn, [] (char ch) { return ch != ';'; }) )
   {
-    if ( *p < '0' || *p > '9')
-    {
-      clearEvent();
-      urxvt_mouse[0] = '\0';  // Delete already interpreted data
-      return;
-    }
-
-    btn = 10 * btn + (*p - '0');
-    p++;
+    clearEvent();
+    urxvt_mouse[0] = '\0';  // Delete already interpreted data
+    return;
   }
 
   if ( *++p == '-' )
@@ -1064,17 +1083,12 @@ void FMouseUrxvt::processEvent (const TimeValue& time)
     x_neg = true;
   }
 
-  while ( *p && *p != ';' )
+  // Parse x-value
+  if ( ! parseNumberIf (p, x, [] (char ch) { return ch != ';'; }) )
   {
-    if ( *p < '0' || *p > '9')
-    {
-      clearEvent();
-      urxvt_mouse[0] = '\0';  // Delete already interpreted data
-      return;
-    }
-
-    x = uInt16(10 * x + (*p - '0'));
-    p++;
+    clearEvent();
+    urxvt_mouse[0] = '\0';  // Delete already interpreted data
+    return;
   }
 
   if ( *++p == '-' )
@@ -1083,17 +1097,12 @@ void FMouseUrxvt::processEvent (const TimeValue& time)
     y_neg = true;
   }
 
-  while ( *p && *p != 'M' )
+  // Parse y-value
+  if ( ! parseNumberIf (p, y, [] (char ch) { return ch != 'M'; }) )
   {
-    if ( *p < '0' || *p > '9')
-    {
-      clearEvent();
-      urxvt_mouse[0] = '\0';  // Delete already interpreted data
-      return;
-    }
-
-    y = uInt16(10 * y + (*p - '0'));
-    p++;
+    clearEvent();
+    urxvt_mouse[0] = '\0';  // Delete already interpreted data
+    return;
   }
 
   if ( x_neg || x == 0 )
@@ -1102,12 +1111,8 @@ void FMouseUrxvt::processEvent (const TimeValue& time)
   if ( y_neg || y == 0 )
     y = 1;
 
-  if ( x > getMaxWidth() )
-    x = getMaxWidth();
-
-  if ( y > getMaxHeight() )
-    y = getMaxHeight();
-
+  x = std::min(x, sInt16(getMaxWidth()));
+  y = std::min(y, sInt16(getMaxHeight()));
   setNewPos (x, y);
   clearButtonState();
   setKeyState (btn);
@@ -1125,7 +1130,7 @@ void FMouseUrxvt::processEvent (const TimeValue& time)
   }
 
   setEvent();
-  setPos (FPoint{x, y});
+  useNewPos();
   urxvt_button_state = uChar(btn);
   // Delete already interpreted data
   urxvt_mouse[0] = '\0';
@@ -1195,26 +1200,7 @@ void FMouseUrxvt::setButtonState (const int btn, const TimeValue& time) noexcept
       break;
 
     case all_buttons_released:
-      switch ( urxvt_button_state & button_mask )
-      {
-        case button1_pressed:
-        case button1_pressed_move:
-          getButtonState().left_button = State::Released;
-          break;
-
-        case button2_pressed:
-        case button2_pressed_move:
-          getButtonState().middle_button = State::Released;
-          break;
-
-        case button3_pressed:
-        case button3_pressed_move:
-          getButtonState().right_button = State::Released;
-          break;
-
-        default:
-          break;
-      }
+      handleButtonRelease();
       break;
 
     case button_up:
@@ -1232,6 +1218,31 @@ void FMouseUrxvt::setButtonState (const int btn, const TimeValue& time) noexcept
   }
 }
 
+//----------------------------------------------------------------------
+void FMouseUrxvt::handleButtonRelease() noexcept
+{
+  switch ( urxvt_button_state & button_mask )
+  {
+    case button1_pressed:
+    case button1_pressed_move:
+      getButtonState().left_button = State::Released;
+      break;
+
+    case button2_pressed:
+    case button2_pressed_move:
+      getButtonState().middle_button = State::Released;
+      break;
+
+    case button3_pressed:
+    case button3_pressed_move:
+      getButtonState().right_button = State::Released;
+      break;
+
+    default:
+      break;
+  }
+}
+
 
 //----------------------------------------------------------------------
 // class FMouseControl
@@ -1259,12 +1270,12 @@ FMouseControl::~FMouseControl() = default;  // destructor
 //----------------------------------------------------------------------
 auto FMouseControl::getInstance() -> FMouseControl&
 {
-  static const auto& mouse = make_unique<FMouseControl>();
+  static const auto& mouse = std::make_unique<FMouseControl>();
   return *mouse;
 }
 
 //----------------------------------------------------------------------
-const FPoint& FMouseControl::getPos() &
+auto FMouseControl::getPos() & -> const FPoint&
 {
   const auto& iter = findMouseWithEvent();
 
@@ -1344,19 +1355,19 @@ void FMouseControl::useXtermMouse (bool enable)
 }
 
 //----------------------------------------------------------------------
-bool FMouseControl::hasData()
+auto FMouseControl::hasData() -> bool
 {
   return findMouseWithData() != mouse_protocol.end();  // with data
 }
 
 //----------------------------------------------------------------------
-bool FMouseControl::hasEvent()
+auto FMouseControl::hasEvent() -> bool
 {
   return findMouseWithEvent() != mouse_protocol.end();  // with event
 }
 
 //----------------------------------------------------------------------
-bool FMouseControl::isLeftButtonPressed()
+auto FMouseControl::isLeftButtonPressed() -> bool
 {
   auto iter = findMouseWithEvent();
   const bool found = (iter != mouse_protocol.end());
@@ -1364,7 +1375,7 @@ bool FMouseControl::isLeftButtonPressed()
 }
 
 //----------------------------------------------------------------------
-bool FMouseControl::isLeftButtonReleased()
+auto FMouseControl::isLeftButtonReleased() -> bool
 {
   auto iter = findMouseWithEvent();
   const bool found = (iter != mouse_protocol.end());
@@ -1372,7 +1383,7 @@ bool FMouseControl::isLeftButtonReleased()
 }
 
 //----------------------------------------------------------------------
-bool FMouseControl::isLeftButtonDoubleClick()
+auto FMouseControl::isLeftButtonDoubleClick() -> bool
 {
   auto iter = findMouseWithEvent();
   const bool found = (iter != mouse_protocol.end());
@@ -1380,7 +1391,7 @@ bool FMouseControl::isLeftButtonDoubleClick()
 }
 
 //----------------------------------------------------------------------
-bool FMouseControl::isRightButtonPressed()
+auto FMouseControl::isRightButtonPressed() -> bool
 {
   auto iter = findMouseWithEvent();
   const bool found = (iter != mouse_protocol.end());
@@ -1388,7 +1399,7 @@ bool FMouseControl::isRightButtonPressed()
 }
 
 //----------------------------------------------------------------------
-bool FMouseControl::isRightButtonReleased()
+auto FMouseControl::isRightButtonReleased() -> bool
 {
   auto iter = findMouseWithEvent();
   const bool found = (iter != mouse_protocol.end());
@@ -1396,7 +1407,7 @@ bool FMouseControl::isRightButtonReleased()
 }
 
 //----------------------------------------------------------------------
-bool FMouseControl::isMiddleButtonPressed()
+auto FMouseControl::isMiddleButtonPressed() -> bool
 {
   auto iter = findMouseWithEvent();
   const bool found = (iter != mouse_protocol.end());
@@ -1404,7 +1415,7 @@ bool FMouseControl::isMiddleButtonPressed()
 }
 
 //----------------------------------------------------------------------
-bool FMouseControl::isMiddleButtonReleased()
+auto FMouseControl::isMiddleButtonReleased() -> bool
 {
   auto iter = findMouseWithEvent();
   const bool found = (iter != mouse_protocol.end());
@@ -1412,7 +1423,7 @@ bool FMouseControl::isMiddleButtonReleased()
 }
 
 //----------------------------------------------------------------------
-bool FMouseControl::isShiftKeyPressed()
+auto FMouseControl::isShiftKeyPressed() -> bool
 {
   auto iter = findMouseWithEvent();
   const bool found = (iter != mouse_protocol.end());
@@ -1420,7 +1431,7 @@ bool FMouseControl::isShiftKeyPressed()
 }
 
 //----------------------------------------------------------------------
-bool FMouseControl::isControlKeyPressed()
+auto FMouseControl::isControlKeyPressed() -> bool
 {
   auto iter = findMouseWithEvent();
   const bool found = (iter != mouse_protocol.end());
@@ -1428,7 +1439,7 @@ bool FMouseControl::isControlKeyPressed()
 }
 
 //----------------------------------------------------------------------
-bool FMouseControl::isMetaKeyPressed()
+auto FMouseControl::isMetaKeyPressed() -> bool
 {
   auto iter = findMouseWithEvent();
   const bool found = (iter != mouse_protocol.end());
@@ -1436,7 +1447,7 @@ bool FMouseControl::isMetaKeyPressed()
 }
 
 //----------------------------------------------------------------------
-bool FMouseControl::isWheelUp()
+auto FMouseControl::isWheelUp() -> bool
 {
   auto iter = findMouseWithEvent();
   const bool found = (iter != mouse_protocol.end());
@@ -1444,7 +1455,7 @@ bool FMouseControl::isWheelUp()
 }
 
 //----------------------------------------------------------------------
-bool FMouseControl::isWheelDown()
+auto FMouseControl::isWheelDown() -> bool
 {
   auto iter = findMouseWithEvent();
   const bool found = (iter != mouse_protocol.end());
@@ -1452,7 +1463,7 @@ bool FMouseControl::isWheelDown()
 }
 
 //----------------------------------------------------------------------
-bool FMouseControl::isMoved()
+auto FMouseControl::isMoved() -> bool
 {
   auto iter = findMouseWithEvent();
   const bool found = (iter != mouse_protocol.end());
@@ -1460,11 +1471,11 @@ bool FMouseControl::isMoved()
 }
 
 //----------------------------------------------------------------------
-bool FMouseControl::hasUnprocessedInput()
+auto FMouseControl::hasUnprocessedInput() const -> bool
 {
-  return std::any_of ( std::begin(mouse_protocol)
-                     , std::end(mouse_protocol)
-                     , [] (FMouseProtocol::const_reference mouse)
+  return std::any_of ( std::cbegin(mouse_protocol)
+                     , std::cend(mouse_protocol)
+                     , [] (const auto& mouse)
                        {
                          return mouse->hasUnprocessedInput();
                        }
@@ -1473,7 +1484,7 @@ bool FMouseControl::hasUnprocessedInput()
 
 //----------------------------------------------------------------------
 #ifdef F_HAVE_LIBGPM
-bool FMouseControl::isGpmMouseEnabled() noexcept
+auto FMouseControl::isGpmMouseEnabled() noexcept -> bool
 {
   if ( ! use_gpm_mouse || mouse_protocol.empty() )
     return false;
@@ -1527,7 +1538,7 @@ void FMouseControl::disable()
 
 //----------------------------------------------------------------------
 void FMouseControl::setRawData ( const FMouse::MouseType& mt
-                               , FKeyboard::keybuffer& fifo_buf)
+                               , FKeyboard::keybuffer& fifo_buf )
 {
   const auto iter = findMouseWithType(mt);
 
@@ -1538,7 +1549,7 @@ void FMouseControl::setRawData ( const FMouse::MouseType& mt
 //----------------------------------------------------------------------
 void FMouseControl::processQueuedInput()
 {
-  while ( ! fmousedata_queue.empty() )
+  while ( ! fmousedata_queue.isEmpty() )
   {
     if ( FApplication::isQuit() )
       return;
@@ -1548,9 +1559,9 @@ void FMouseControl::processQueuedInput()
 
     if ( md )
     {
-      current_mouse_event = md.get();
+      setCurrentMouseEvent (md);
       event_cmd.execute(*md);
-      current_mouse_event = nullptr;
+      resetCurrentMouseEvent();
     }
 
     if ( FApplication::isQuit() )
@@ -1570,13 +1581,13 @@ void FMouseControl::processEvent (const TimeValue& time)
   {
     (*iter)->processEvent(time);
     auto& md = static_cast<FMouseData&>(**iter);
-    fmousedata_queue.emplace(make_unique<FMouseData>(std::move(md)));
+    fmousedata_queue.emplace(std::make_unique<FMouseData>(std::move(md)));
   }
 }
 
 //----------------------------------------------------------------------
 #ifdef F_HAVE_LIBGPM
-bool FMouseControl::getGpmKeyPressed (bool pending)
+auto FMouseControl::getGpmKeyPressed (bool pending) -> bool
 {
   if ( mouse_protocol.empty() )
     return false;
@@ -1613,11 +1624,11 @@ void FMouseControl::drawPointer()
 
 // private methods of FMouseControl
 //----------------------------------------------------------------------
-auto FMouseControl::findMouseWithType (const FMouse::MouseType& mt) -> FMouseProtocol::const_iterator
+auto FMouseControl::findMouseWithType (const FMouse::MouseType& mt) const -> FMouseProtocol::const_iterator
 {
-  return std::find_if ( std::begin(mouse_protocol)
-                      , std::end(mouse_protocol)
-                      , [&mt] (FMouseProtocol::const_reference mouse)
+  return std::find_if ( std::cbegin(mouse_protocol)
+                      , std::cend(mouse_protocol)
+                      , [&mt] (const auto& mouse)
                         {
                           return mouse->getMouseTypeID() == mt;
                         }
@@ -1625,11 +1636,11 @@ auto FMouseControl::findMouseWithType (const FMouse::MouseType& mt) -> FMousePro
 }
 
 //----------------------------------------------------------------------
-auto FMouseControl::findMouseWithData() -> FMouseProtocol::const_iterator
+auto FMouseControl::findMouseWithData() const -> FMouseProtocol::const_iterator
 {
-  return std::find_if ( std::begin(mouse_protocol)
-                      , std::end(mouse_protocol)
-                      , [] (FMouseProtocol::const_reference mouse)
+  return std::find_if ( std::cbegin(mouse_protocol)
+                      , std::cend(mouse_protocol)
+                      , [] (const auto& mouse)
                         {
                           return mouse->hasData();
                         }
@@ -1637,11 +1648,11 @@ auto FMouseControl::findMouseWithData() -> FMouseProtocol::const_iterator
 }
 
 //----------------------------------------------------------------------
-auto FMouseControl::findMouseWithEvent() -> FMouseProtocol::const_iterator
+auto FMouseControl::findMouseWithEvent() const -> FMouseProtocol::const_iterator
 {
-  return std::find_if ( std::begin(mouse_protocol)
-                      , std::end(mouse_protocol)
-                      , [] (FMouseProtocol::const_reference mouse)
+  return std::find_if ( std::cbegin(mouse_protocol)
+                      , std::cend(mouse_protocol)
+                      , [] (const auto& mouse)
                         {
                           return mouse->hasEvent();
                         }
@@ -1659,4 +1670,16 @@ void FMouseControl::xtermMouse (bool enable) const
   FTermXTerminal::setMouseSupport (enable);
 }
 
+//----------------------------------------------------------------------
+void FMouseControl::setCurrentMouseEvent (const FMouseDataPtr& ptr)
+{
+  getCurrentMouseEvent() = ptr;
+}
+
+//----------------------------------------------------------------------
+void FMouseControl::resetCurrentMouseEvent()
+{
+  getCurrentMouseEvent() = nullptr;
+}
+
 }  // namespace finalcut
