@@ -3,7 +3,7 @@
 *                                                                      *
 * This file is part of the FINAL CUT widget toolkit                    *
 *                                                                      *
-* Copyright 2015-2021 Markus Gans                                      *
+* Copyright 2015-2023 Markus Gans                                      *
 *                                                                      *
 * FINAL CUT is free software; you can redistribute it and/or modify    *
 * it under the terms of the GNU Lesser General Public License as       *
@@ -76,7 +76,7 @@ FMenuItem::~FMenuItem()  // destructor
       menu_list->remove(this);
   }
 
-  delAccelerator();
+  FMenuItem::delAccelerator();
 
   // remove dialog list item callback from the dialog
   if ( associated_window )
@@ -86,7 +86,7 @@ FMenuItem::~FMenuItem()  // destructor
 
 // public methods of FMenuItem
 //----------------------------------------------------------------------
-bool FMenuItem::setEnable (bool enable)
+auto FMenuItem::setEnable (bool enable) -> bool
 {
   FWidget::setEnable(enable);
   auto super = getSuperMenu();
@@ -109,46 +109,6 @@ bool FMenuItem::setEnable (bool enable)
   return enable;
 }
 
-//----------------------------------------------------------------------
-bool FMenuItem::setFocus (bool enable)
-{
-  FWidget::setFocus(enable);
-
-  if ( ! enable || ! isEnabled() || selected )
-    return enable;
-
-  auto menu_list = getFMenuList(*getSuperMenu());
-  setSelected();
-
-  if ( menu_list )
-  {
-    menu_list->unselectItem();
-    menu_list->setSelectedItem(this);
-  }
-
-  if ( getStatusBar() )
-    getStatusBar()->drawMessage();
-
-  auto parent = getSuperMenu();
-
-  if ( isMenuBar(parent) )
-  {
-    auto menubar_ptr = static_cast<FMenuBar*>(parent);
-
-    if ( menubar_ptr )
-      menubar_ptr->redraw();
-  }
-  else if ( isMenu(parent) )
-  {
-    auto menu_ptr = static_cast<FMenu*>(parent);
-
-    if ( menu_ptr )
-      menu_ptr->redraw();
-  }
-
-  return enable;
-}
-
 //----------------------------------------------------------------------
 void FMenuItem::setSelected()
 {
@@ -178,7 +138,7 @@ void FMenuItem::setText (const FString& txt)
 void FMenuItem::addAccelerator (FKey key, FWidget* obj) &
 {
   const auto& root = getRootWidget();
-  FAccelerator accel = { key, obj };
+  const FAccelerator accel{ key, obj };
 
   if ( root )
   {
@@ -196,7 +156,7 @@ void FMenuItem::delAccelerator (FWidget* obj) &
 
   if ( root && ! root->setAcceleratorList().empty() )
   {
-    auto list = root->setAcceleratorList();
+    auto& list = root->setAcceleratorList();
     auto iter = list.cbegin();
 
     while ( iter != list.cend() )
@@ -384,9 +344,7 @@ void FMenuItem::onAccel (FAccelEvent* ev)
 
   if ( menu )
   {
-    if ( mbar->getSelectedItem() )
-      mbar->getSelectedItem()->unsetSelected();
-
+    resetSelectedItem(mbar);
     setSelected();
     mbar->setSelectedItem(this);
     openMenu();
@@ -401,10 +359,7 @@ void FMenuItem::onAccel (FAccelEvent* ev)
       focused_widget->redraw();
 
     menu->redraw();
-
-    if ( getStatusBar() )
-      getStatusBar()->drawMessage();
-
+    drawStatusBarMessage();
     mbar->redraw();
     mbar->drop_down = true;
   }
@@ -422,46 +377,70 @@ void FMenuItem::onAccel (FAccelEvent* ev)
 }
 
 //----------------------------------------------------------------------
-void FMenuItem::onFocusIn (FFocusEvent*)
+void FMenuItem::onFocusIn (FFocusEvent* in_ev)
 {
-  if ( getStatusBar() )
-    getStatusBar()->drawMessage();
+  if ( selected )
+  {
+    FWidget::onFocusIn(in_ev);
+    return;
+  }
+
+  auto menu_list = getFMenuList(*getSuperMenu());
+  auto parent = getSuperMenu();
+  setSelected();
+
+  if ( menu_list )
+  {
+    menu_list->unselectItem();
+    menu_list->setSelectedItem(this);
+  }
+
+  if ( isMenuBar(parent) )
+  {
+    auto menubar_ptr = static_cast<FMenuBar*>(parent);
+
+    if ( menubar_ptr )
+      menubar_ptr->redraw();
+  }
+  else if ( isMenu(parent) )
+  {
+    auto menu_ptr = static_cast<FMenu*>(parent);
+
+    if ( menu_ptr )
+      menu_ptr->redraw();
+  }
+
+  FWidget::onFocusIn(in_ev);
 }
 
 //----------------------------------------------------------------------
-void FMenuItem::onFocusOut (FFocusEvent*)
+void FMenuItem::onFocusOut (FFocusEvent* out_ev)
 {
-  unsetSelected();
-
-  if ( super_menu && isMenuBar(super_menu) )
+  if ( isMenuBar(super_menu) )
   {
     auto mbar = static_cast<FMenuBar*>(super_menu);
     mbar->redraw();
   }
 
-  if ( getStatusBar() )
-  {
-    getStatusBar()->clearMessage();
-    getStatusBar()->drawMessage();
-  }
+  FWidget::onFocusOut(out_ev);
 }
 
 
 // protected methods of FMenuItem
 //----------------------------------------------------------------------
-bool FMenuItem::isDialog (const FWidget* w) const
+auto FMenuItem::isDialog (const FWidget* w) const -> bool
 {
   return w ? w->isDialogWidget() : false;
 }
 
 //----------------------------------------------------------------------
-bool FMenuItem::isMenuBar (const FWidget* w) const
+auto FMenuItem::isMenuBar (const FWidget* w) const -> bool
 {
   return w ? w->isInstanceOf("FMenuBar") : false;
 }
 
 //----------------------------------------------------------------------
-bool FMenuItem::isMenu (const FWidget* w) const
+auto FMenuItem::isMenu (const FWidget* w) const -> bool
 {
   if ( ! w )
     return false;
@@ -479,7 +458,7 @@ void FMenuItem::initLayout()
 
 // private methods of FMenuItem
 //----------------------------------------------------------------------
-FMenuList* FMenuItem::getFMenuList (FWidget& widget)
+auto FMenuItem::getFMenuList (FWidget& widget) -> FMenuList*
 {
   FMenuList* menu_list{nullptr};
 
@@ -513,8 +492,8 @@ void FMenuItem::init()
     text_width--;
   }
 
-  setGeometry (FPoint{1, 1}, FSize{text_width + 2, 1}, false);
-  FWidget* parent = getParentWidget();
+  FWidget::setGeometry (FPoint{1, 1}, FSize{text_width + 2, 1}, false);
+  const auto& parent = getParentWidget();
 
   if ( ! parent )
     return;
@@ -582,7 +561,7 @@ void FMenuItem::updateSuperMenuDimensions()
 //----------------------------------------------------------------------
 void FMenuItem::updateMenubarDimensions() const
 {
-  FWidget* parent = getParentWidget();
+  const auto& parent = getParentWidget();
 
   if ( ! parent || ! isMenuBar(parent) )
     return;
@@ -650,7 +629,7 @@ void FMenuItem::createDialogList (FMenu* winmenu) const
       win_item->addCallback
       (
         "clicked",
-        static_cast<std::remove_reference<decltype(win_item)>::type>(win_item),
+        static_cast<std::remove_reference_t<decltype(win_item)>>(win_item),
         &FMenuItem::cb_switchToDialog,
         win
       );
@@ -658,7 +637,7 @@ void FMenuItem::createDialogList (FMenu* winmenu) const
       win->addCallback
       (
         "destroy",
-        static_cast<std::remove_reference<decltype(win_item)>::type>(win_item),
+        static_cast<std::remove_reference_t<decltype(win_item)>>(win_item),
         &FMenuItem::cb_destroyDialog,
         win
       );
@@ -702,6 +681,18 @@ void FMenuItem::passMouseEvent ( T widget, const FMouseEvent* ev
   }
 }
 
+//----------------------------------------------------------------------
+inline void FMenuItem::resetSelectedItem (const FMenuList* list) const
+{
+  if ( ! list )
+    return;
+
+  auto selected_item = list->getSelectedItem();
+
+  if ( selected_item )
+    selected_item->unsetSelected();
+}
+
 //----------------------------------------------------------------------
 void FMenuItem::cb_switchToDialog (FDialog* win) const
 {
