@@ -3,7 +3,7 @@
 *                                                                      *
 * This file is part of the FINAL CUT widget toolkit                    *
 *                                                                      *
-* Copyright 2015-2021 Markus Gans                                      *
+* Copyright 2015-2023 Markus Gans                                      *
 *                                                                      *
 * FINAL CUT is free software; you can redistribute it and/or modify    *
 * it under the terms of the GNU Lesser General Public License as       *
@@ -20,8 +20,10 @@
 * <http://www.gnu.org/licenses/>.                                      *
 ***********************************************************************/
 
-#include <cstring>
+#include <algorithm>
 #include <array>
+#include <cstring>
+#include <memory>
 
 #include "final/fapplication.h"
 #include "final/fc.h"
@@ -57,7 +59,7 @@ FOptiMove::FOptiMove (int baud)
 //----------------------------------------------------------------------
 auto FOptiMove::getInstance() -> FOptiMove&
 {
-  static const auto& opti_move = make_unique<FOptiMove>();
+  static const auto& opti_move = std::make_unique<FOptiMove>();
   return *opti_move;
 }
 
@@ -474,21 +476,14 @@ void FOptiMove::check_boundaries ( int& xold, int& yold
   if ( yold < 0 || yold >= int(screen_height) )
     yold = -1;
 
-  if ( xnew < 0 )
-    xnew = 0;
-
-  if ( ynew < 0 )
-    ynew = 0;
-
-  if ( xnew >= int(screen_width) )
-    xnew = int(screen_width) - 1;
-
-  if ( ynew >= int(screen_height) )
-    ynew = int(screen_height) - 1;
+  xnew = std::max(0, xnew);
+  ynew = std::max(0, ynew);
+  xnew = std::min(int(screen_width) - 1, xnew);
+  ynew = std::min(int(screen_height) - 1, ynew);
 }
 
 //----------------------------------------------------------------------
-std::string FOptiMove::moveCursor (int xold, int yold, int xnew, int ynew)
+auto FOptiMove::moveCursor (int xold, int yold, int xnew, int ynew) -> std::string
 {
   int method{0};
   int move_time{LONG_DURATION};
@@ -552,7 +547,7 @@ void FOptiMove::calculateCharDuration()
 }
 
 //----------------------------------------------------------------------
-int FOptiMove::capDuration (const char cap[], int affcnt) const
+auto FOptiMove::capDuration (const char cap[], int affcnt) const -> int
 {
   // calculate the duration in milliseconds of a given operation
   // cap    - the term capability
@@ -564,15 +559,8 @@ int FOptiMove::capDuration (const char cap[], int affcnt) const
   float ms{0};
   const char* p = cap;
 
-  while ( *p )
-  {
-    // check for delay with padding character
-    if ( p[0] == '$' && p[1] == '<' && std::strchr(p, '>') )
-    {
-      float num = 0;
-      p += 2;
-
-      while ( *p != '>' )
+  auto parse_digit = \
+      [&p, &affcnt] (float& num)
       {
         if ( std::isdigit(uChar(*p)) )
           num = num * 10 + float(*p - '0');
@@ -585,7 +573,19 @@ int FOptiMove::capDuration (const char cap[], int affcnt) const
           if ( *p != '>' && std::isdigit(uChar(*p)) )
             num += float((*p - '0') / 10.0);
         }
+      };
 
+  while ( *p )
+  {
+    // check for delay with padding character
+    if ( p[0] == '$' && p[1] == '<' && std::strchr(p, '>') )
+    {
+      float num = 0;
+      p += 2;
+
+      while ( *p != '>' )
+      {
+        parse_digit(num);
         p++;
       }
 
@@ -601,7 +601,7 @@ int FOptiMove::capDuration (const char cap[], int affcnt) const
 }
 
 //----------------------------------------------------------------------
-int FOptiMove::capDurationToLength (int duration) const
+auto FOptiMove::capDurationToLength (int duration) const -> int
 {
   if ( duration != LONG_DURATION )
     return (duration + char_duration - 1) / char_duration;
@@ -610,12 +610,12 @@ int FOptiMove::capDurationToLength (int duration) const
 }
 
 //----------------------------------------------------------------------
-int FOptiMove::repeatedAppend ( std::string& dst
-                              , const Capability& o
-                              , int count ) const
+auto FOptiMove::repeatedAppend ( std::string& dst
+                               , const Capability& o
+                               , int count ) const -> int
 {
-  const std::size_t src_len = stringLength(o.cap);
-  const std::size_t dst_len = dst.length();
+  const auto& src_len = stringLength(o.cap);
+  const auto& dst_len = dst.length();
   int total{0};
 
   if ( (dst_len + uInt(count) * src_len) < BUF_SIZE - 1 )
@@ -636,9 +636,9 @@ int FOptiMove::repeatedAppend ( std::string& dst
 }
 
 //----------------------------------------------------------------------
-int FOptiMove::relativeMove ( std::string& move
-                            , int from_x, int from_y
-                            , int to_x, int to_y ) const
+auto FOptiMove::relativeMove ( std::string& move
+                             , int from_x, int from_y
+                             , int to_x, int to_y ) const -> int
 {
   int vtime{0};
   int htime{0};
@@ -672,7 +672,7 @@ int FOptiMove::relativeMove ( std::string& move
 }
 
 //----------------------------------------------------------------------
-inline int FOptiMove::verticalMove (std::string& move, int from_y, int to_y) const
+inline auto FOptiMove::verticalMove (std::string& move, int from_y, int to_y) const -> int
 {
   int vtime{LONG_DURATION};
 
@@ -734,7 +734,7 @@ inline void FOptiMove::upMove ( std::string& move, int& vtime
 }
 
 //----------------------------------------------------------------------
-inline int FOptiMove::horizontalMove (std::string& hmove, int from_x, int to_x) const
+inline auto FOptiMove::horizontalMove (std::string& hmove, int from_x, int to_x) const -> int
 {
   int htime{LONG_DURATION};
 
@@ -753,53 +753,111 @@ inline int FOptiMove::horizontalMove (std::string& hmove, int from_x, int to_x)
   return htime;
 }
 
+//----------------------------------------------------------------------
+inline void FOptiMove::moveWithParmRightCursor ( std::string& hmove
+                                               , int& htime, int num ) const
+{
+  // Use parameterized cursor right capability
+  hmove = FTermcap::encodeParameter(F_parm_right_cursor.cap, num);
+  htime = F_parm_right_cursor.duration;
+}
+
+//----------------------------------------------------------------------
+inline void FOptiMove::moveWithRightCursor ( std::string& hmove, int& htime
+                                           , int num, int from_x, int to_x ) const
+{
+  std::string str{};
+  int htime_r{0};
+
+  // try to use tab
+  if ( tabstop > 0 && F_tab.cap )
+  {
+    int pos = from_x;
+
+    for ( int tab_pos = pos + tabstop - (pos % tabstop)
+        ; tab_pos <= to_x
+        ; tab_pos += tabstop )
+    {
+      htime_r += repeatedAppend (str, F_tab, 1);
+
+      if ( htime_r >= LONG_DURATION )
+        break;
+
+      pos = tab_pos;
+    }
+
+    num = to_x - pos;
+  }
+
+  // Use the cursor right capability
+  htime_r += repeatedAppend (str, F_cursor_right, num);
+
+  if ( htime_r < htime )
+  {
+    hmove = str;
+    htime = htime_r;
+  }
+}
+
 //----------------------------------------------------------------------
 inline void FOptiMove::rightMove ( std::string& hmove, int& htime
                                  , int from_x, int to_x ) const
 {
   int num = to_x - from_x;
 
+  if ( num == 0 )
+    return;
+
   if ( F_parm_right_cursor.cap && F_parm_right_cursor.duration < htime )
-  {
-    hmove = FTermcap::encodeParameter(F_parm_right_cursor.cap, num);
-    htime = F_parm_right_cursor.duration;
-  }
+    moveWithParmRightCursor (hmove, htime, num);
 
   if ( F_cursor_right.cap )
+    moveWithRightCursor (hmove, htime, num, from_x, to_x);
+}
+
+//----------------------------------------------------------------------
+inline void FOptiMove::moveWithParmLeftCursor ( std::string& hmove
+                                              , int& htime, int num ) const
+{
+  // Use parameterized cursor right capability
+  hmove = FTermcap::encodeParameter(F_parm_left_cursor.cap, num);
+  htime = F_parm_left_cursor.duration;
+}
+
+//----------------------------------------------------------------------
+inline void FOptiMove::moveWithLeftCursor ( std::string& hmove, int& htime
+                                          , int num, int from_x, int to_x ) const
+{
+  std::string str{};
+  int htime_l{0};
+
+  // try to use backward tab
+  if ( tabstop > 0 && F_back_tab.cap )
   {
-    std::string str{};
-    int htime_r{0};
+    int pos = from_x;
 
-    // try to use tab
-    if ( tabstop > 0 && F_tab.cap )
+    for ( int tab_pos = ( pos > 0 ) ? ((pos - 1) / tabstop) * tabstop : -1
+        ; tab_pos >= to_x
+        ; tab_pos = ( pos > 0 ) ? ((pos - 1) / tabstop) * tabstop : -1)
     {
-      int pos = from_x;
+      htime_l += repeatedAppend (str, F_back_tab, 1);
 
-      while ( true )
-      {
-        const int tab_pos = pos + tabstop - (pos % tabstop);
+      if ( htime_l >= LONG_DURATION )
+        break;
 
-        if ( tab_pos > to_x )
-          break;
-
-        htime_r += repeatedAppend (str, F_tab, 1);
-
-        if ( htime_r >= LONG_DURATION )
-          break;
-
-        pos = tab_pos;
-      }
-
-      num = to_x - pos;
+      pos = tab_pos;
     }
 
-    htime_r += repeatedAppend (str, F_cursor_right, num);
+    num = pos - to_x;
+  }
 
-    if ( htime_r < htime )
-    {
-      hmove = str;
-      htime = htime_r;
-    }
+  // Use the cursor left capability
+  htime_l += repeatedAppend (str, F_cursor_left, num);
+
+  if ( htime_l < htime )
+  {
+    hmove = str;
+    htime = htime_l;
   }
 }
 
@@ -809,53 +867,19 @@ inline void FOptiMove::leftMove ( std::string& hmove, int& htime
 {
   int num = from_x - to_x;
 
+  if ( num == 0 )
+    return;
+
   if ( F_parm_left_cursor.cap && F_parm_left_cursor.duration < htime )
-  {
-    hmove = FTermcap::encodeParameter(F_parm_left_cursor.cap, num);
-    htime = F_parm_left_cursor.duration;
-  }
+    moveWithParmLeftCursor (hmove, htime, num);
 
   if ( F_cursor_left.cap )
-  {
-    std::string str{};
-    int htime_l{0};
-
-    // try to use backward tab
-    if ( tabstop > 0 && F_back_tab.cap )
-    {
-      int pos = from_x;
-
-      while ( true )
-      {
-        const int tab_pos = ( pos > 0 ) ? ((pos - 1) / tabstop) * tabstop : -1;
-
-        if ( tab_pos < to_x )
-          break;
-
-        htime_l += repeatedAppend (str, F_back_tab, 1);
-
-        if ( htime_l >= LONG_DURATION )
-          break;
-
-        pos = tab_pos;
-      }
-
-      num = pos - to_x;
-    }
-
-    htime_l += repeatedAppend (str, F_cursor_left, num);
-
-    if ( htime_l < htime )
-    {
-      hmove = str;
-      htime = htime_l;
-    }
-  }
+    moveWithLeftCursor (hmove, htime, num, from_x, to_x);
 }
 
 //----------------------------------------------------------------------
-inline bool FOptiMove::isWideMove ( int xold, int yold
-                                  , int xnew, int ynew ) const
+inline auto FOptiMove::isWideMove ( int xold, int yold
+                                  , int xnew, int ynew ) const -> bool
 {
   return xnew > MOVE_LIMIT
       && xnew < int(screen_width) - 1 - MOVE_LIMIT
@@ -863,8 +887,8 @@ inline bool FOptiMove::isWideMove ( int xold, int yold
 }
 
 //----------------------------------------------------------------------
-inline bool FOptiMove::isMethod0Faster ( int& move_time
-                                       , int xnew, int ynew )
+inline auto FOptiMove::isMethod0Faster ( int& move_time
+                                       , int xnew, int ynew ) -> bool
 {
   // Test method 0: direct cursor addressing
 
@@ -885,16 +909,15 @@ inline bool FOptiMove::isMethod0Faster ( int& move_time
 }
 
 //----------------------------------------------------------------------
-inline bool FOptiMove::isMethod1Faster ( int& move_time
+inline auto FOptiMove::isMethod1Faster ( int& move_time
                                        , int xold, int yold
-                                       , int xnew, int ynew ) const
+                                       , int xnew, int ynew ) -> bool
 {
   // Test method 1: local movement
 
   if ( xold >= 0 && yold >= 0 )
   {
-    std::string null_result{};
-    const int new_time = relativeMove (null_result, xold, yold, xnew, ynew);
+    const int new_time = relativeMove (temp_result, xold, yold, xnew, ynew);
 
     if ( new_time < LONG_DURATION && new_time < move_time )
     {
@@ -907,16 +930,15 @@ inline bool FOptiMove::isMethod1Faster ( int& move_time
 }
 
 //----------------------------------------------------------------------
-inline bool FOptiMove::isMethod2Faster ( int& move_time
+inline auto FOptiMove::isMethod2Faster ( int& move_time
                                        , int yold
-                                       , int xnew, int ynew ) const
+                                       , int xnew, int ynew ) -> bool
 {
   // Test method 2: carriage-return + local movement
 
   if ( yold >= 0 && F_carriage_return.cap )
   {
-    std::string null_result{};
-    const int new_time = relativeMove (null_result, 0, yold, xnew, ynew);
+    const int new_time = relativeMove (temp_result, 0, yold, xnew, ynew);
 
     if ( new_time < LONG_DURATION
       && F_carriage_return.duration + new_time < move_time )
@@ -930,15 +952,14 @@ inline bool FOptiMove::isMethod2Faster ( int& move_time
 }
 
 //----------------------------------------------------------------------
-inline bool FOptiMove::isMethod3Faster ( int& move_time
-                                       , int xnew, int ynew ) const
+inline auto FOptiMove::isMethod3Faster ( int& move_time
+                                       , int xnew, int ynew ) -> bool
 {
   // Test method 3: home-cursor + local movement
 
   if ( F_cursor_home.cap )
   {
-    std::string null_result{};
-    const int new_time = relativeMove (null_result, 0, 0, xnew, ynew);
+    const int new_time = relativeMove (temp_result, 0, 0, xnew, ynew);
 
     if ( new_time < LONG_DURATION
       && F_cursor_home.duration + new_time < move_time )
@@ -952,15 +973,14 @@ inline bool FOptiMove::isMethod3Faster ( int& move_time
 }
 
 //----------------------------------------------------------------------
-inline bool FOptiMove::isMethod4Faster ( int& move_time
-                                       , int xnew, int ynew ) const
+inline auto FOptiMove::isMethod4Faster ( int& move_time
+                                       , int xnew, int ynew ) -> bool
 {
   // Test method 4: home-down + local movement
   if ( F_cursor_to_ll.cap )
   {
-    std::string null_result{};
     int down = int(screen_height) - 1;
-    const int new_time = relativeMove (null_result, 0, down, xnew, ynew);
+    const int new_time = relativeMove (temp_result, 0, down, xnew, ynew);
 
     if ( new_time < LONG_DURATION
       && F_cursor_to_ll.duration + new_time < move_time )
@@ -974,9 +994,9 @@ inline bool FOptiMove::isMethod4Faster ( int& move_time
 }
 
 //----------------------------------------------------------------------
-inline bool FOptiMove::isMethod5Faster ( int& move_time
+inline auto FOptiMove::isMethod5Faster ( int& move_time
                                        , int yold
-                                       , int xnew, int ynew ) const
+                                       , int xnew, int ynew ) -> bool
 {
   // Test method 5: left margin for wrap to right-hand side
   if ( automatic_left_margin
@@ -984,10 +1004,9 @@ inline bool FOptiMove::isMethod5Faster ( int& move_time
     && yold > 0
     && F_cursor_left.cap )
   {
-    std::string null_result{};
     int x = int(screen_width) - 1;
     int y = yold - 1;
-    const int new_time = relativeMove (null_result, x, y, xnew, ynew);
+    const int new_time = relativeMove (temp_result, x, y, xnew, ynew);
 
     if ( new_time < LONG_DURATION
       && F_carriage_return.cap
@@ -1010,6 +1029,9 @@ void FOptiMove::moveByMethod ( int method
 {
   switch ( method )
   {
+    case 0:  // direct cursor addressing
+      return;
+
     case 1:
       relativeMove (move_buf, xold, yold, xnew, ynew);
       break;
@@ -1018,28 +1040,25 @@ void FOptiMove::moveByMethod ( int method
       if ( F_carriage_return.cap )
       {
         move_buf = F_carriage_return.cap;
-        std::string s{};
-        relativeMove (s, 0, yold, xnew, ynew);
-        move_buf.append(s);
+        relativeMove (temp_result, 0, yold, xnew, ynew);
+        move_buf.append(temp_result);
       }
       break;
 
     case 3:
       move_buf = F_cursor_home.cap;
       {
-        std::string s{};
-        relativeMove (s, 0, 0, xnew, ynew);
-        move_buf.append(s);
+        relativeMove (temp_result, 0, 0, xnew, ynew);
+        move_buf.append(temp_result);
       }
       break;
 
     case 4:
       move_buf = F_cursor_to_ll.cap;
       {
-        std::string s{};
         int down = int(screen_height) - 1;
-        relativeMove (s, 0, down, xnew, ynew);
-        move_buf.append(s);
+        relativeMove (temp_result, 0, down, xnew, ynew);
+        move_buf.append(temp_result);
       }
       break;
 
@@ -1051,16 +1070,15 @@ void FOptiMove::moveByMethod ( int method
 
       move_buf.append(F_cursor_left.cap);
       {
-        std::string s{};
         int x = int(screen_width) - 1;
         int y = yold - 1;
-        relativeMove (s, x, y, xnew, ynew);
-        move_buf.append(s);
+        relativeMove (temp_result, x, y, xnew, ynew);
+        move_buf.append(temp_result);
       }
       break;
 
     default:
-      break;
+      throw std::invalid_argument{"Invalid cursor movement method"};
   }
 }
 
