@@ -3,7 +3,7 @@
 *                                                                      *
 * This file is part of the FINAL CUT widget toolkit                    *
 *                                                                      *
-* Copyright 2018-2021 Markus Gans                                      *
+* Copyright 2018-2023 Markus Gans                                      *
 *                                                                      *
 * FINAL CUT is free software; you can redistribute it and/or modify    *
 * it under the terms of the GNU Lesser General Public License as       *
@@ -21,10 +21,12 @@
 ***********************************************************************/
 
 #if defined(__CYGWIN__)
-  #include "final/fconfig.h"  // includes _GNU_SOURCE for fd_set
+  #include "final/fconfig.h"  // includes _GNU_SOURCE
 #endif
 
 #include <array>
+#include <memory>
+#include <string>
 
 #include "final/fapplication.h"
 #include "final/fc.h"
@@ -33,6 +35,7 @@
 #include "final/output/tty/ftermcap.h"
 #include "final/output/tty/ftermdata.h"
 #include "final/output/tty/ftermdetection.h"
+#include "final/output/tty/fterm_functions.h"
 #include "final/output/tty/fterm.h"
 #include "final/output/tty/ftermios.h"
 #include "final/util/emptyfstring.h"
@@ -58,13 +61,13 @@ namespace finalcut
 //----------------------------------------------------------------------
 auto FTermDetection::getInstance() -> FTermDetection&
 {
-  static const auto& term_detection = make_unique<FTermDetection>();
+  static const auto& term_detection = std::make_unique<FTermDetection>();
   return *term_detection;
 }
 
 //----------------------------------------------------------------------
 #if DEBUG
-const FString& FTermDetection::getAnswerbackString() const &
+auto FTermDetection::getAnswerbackString() const & -> const FString&
 {
   // Get the answerback message that was output after
   // sending the enquiry character (ENQ)
@@ -72,7 +75,7 @@ const FString& FTermDetection::getAnswerbackString() const &
 }
 
 //----------------------------------------------------------------------
-const FString& FTermDetection::getSecDAString() const &
+auto FTermDetection::getSecDAString() const & -> const FString&
 {
   // Get the secondary device attributes (SEC_DA)
   return sec_da;
@@ -136,7 +139,7 @@ void FTermDetection::getSystemTermType()
 }
 
 //----------------------------------------------------------------------
-bool FTermDetection::getTTYtype()
+auto FTermDetection::getTTYtype() -> bool
 {
   // Analyse /etc/ttytype and get the term name
   // ------------------------------------------
@@ -197,7 +200,7 @@ bool FTermDetection::getTTYtype()
 
 #if F_HAVE_GETTTYNAM
 //----------------------------------------------------------------------
-bool FTermDetection::getTTYSFileEntry()
+auto FTermDetection::getTTYSFileEntry() -> bool
 {
   // Analyse /etc/ttys and get the term name (used in BSD Unix)
 
@@ -215,7 +218,7 @@ bool FTermDetection::getTTYSFileEntry()
 
   if ( ttys_entryt )
   {
-    const char* type = ttys_entryt->ty_type;
+    const auto& type = ttys_entryt->ty_type;
 
     if ( type != nullptr )
     {
@@ -269,6 +272,10 @@ void FTermDetection::termtypeAnalysis()
   if ( termtype.left(6) == "mlterm" )
     fterm_data.setTermType (FTermType::mlterm);
 
+  // st - simple terminal
+  if ( termtype.left(11) == "st-256color" )
+    fterm_data.setTermType (FTermType::stterm);
+
   // rxvt
   if ( termtype.left(4) == "rxvt" )
     fterm_data.setTermType (FTermType::rxvt);
@@ -298,6 +305,10 @@ void FTermDetection::termtypeAnalysis()
   // kitty
   if ( termtype.left(11) == "xterm-kitty" )
     fterm_data.setTermType (FTermType::kitty);
+
+  // alacritty
+  if ( termtype.left(9) == "alacritty" )
+    fterm_data.setTermType (FTermType::xterm);
 }
 
 //----------------------------------------------------------------------
@@ -363,11 +374,9 @@ void FTermDetection::detectTerminal()
 }
 
 //----------------------------------------------------------------------
-FString FTermDetection::init_256colorTerminal()
+auto FTermDetection::init_256colorTerminal() -> FString
 {
-  if ( get256colorEnvString() )
-    color256 = true;
-  else if ( termtype.includes("256color") )
+  if ( get256colorEnvString() || termtype.includes("256color") )
     color256 = true;
   else
     color256 = false;
@@ -383,7 +392,7 @@ FString FTermDetection::init_256colorTerminal()
 }
 
 //----------------------------------------------------------------------
-bool FTermDetection::get256colorEnvString()
+auto FTermDetection::get256colorEnvString() -> bool
 {
   // Enable 256 color capabilities
   color_env.string1 = std::getenv("COLORTERM");
@@ -405,7 +414,7 @@ bool FTermDetection::get256colorEnvString()
 }
 
 //----------------------------------------------------------------------
-FString FTermDetection::termtype_256color_quirks()
+auto FTermDetection::termtype_256color_quirks() -> FString
 {
   FString new_termtype{};
   static auto& fterm_data = FTermData::getInstance();
@@ -460,7 +469,7 @@ FString FTermDetection::termtype_256color_quirks()
 }
 
 //----------------------------------------------------------------------
-FString FTermDetection::determineMaxColor (const FString& current_termtype)
+auto FTermDetection::determineMaxColor (const FString& current_termtype) -> FString
 {
   // Determine xterm maximum number of colors via OSC 4
 
@@ -500,42 +509,16 @@ FString FTermDetection::determineMaxColor (const FString& current_termtype)
 }
 
 //----------------------------------------------------------------------
-FString FTermDetection::getXTermColorName (FColor color) const
+auto FTermDetection::getXTermColorName (FColor color) const -> FString
 {
-  FString color_str{""};
-  std::array<char, 30> buf{};
-  fd_set ifds{};
-  struct timeval tv{};
-  const int stdin_no = FTermios::getStdIn();
-
   // get color
   auto index = uInt16(color);
   std::fprintf (stdout, OSC "4;%hu;?" BEL, index);
   std::fflush (stdout);
-  FD_ZERO(&ifds);
-  FD_SET(stdin_no, &ifds);
-  tv.tv_sec  = 0;
-  tv.tv_usec = 150000;  // 150 ms
-
-  // read the terminal answer
-  if ( select (stdin_no + 1, &ifds, nullptr, nullptr, &tv) < 1 )
-    return color_str;
-
-  constexpr auto parse = "\033]4;%10hu;%509[^\n]s";
   std::array<char, 35> temp{};
-  std::size_t pos{0};
-
-  do
-  {
-    std::size_t bytes_free = temp.size() - pos - 1;
-    const ssize_t bytes = read(stdin_no, &temp[pos], bytes_free);
-
-    if ( bytes <= 0 )
-      break;
-
-    pos += std::size_t(bytes);
-  }
-  while ( pos < temp.size() );
+  auto pos = captureTerminalInput(temp, 150'000);
+  constexpr auto parse = "\033]4;%10hu;%509[^\n]s";
+  std::array<char, 30> buf{};
 
   if ( pos > 4 && std::sscanf(temp.data(), parse, &index, buf.data()) == 2 )
   {
@@ -549,14 +532,14 @@ FString FTermDetection::getXTermColorName (FColor color) const
     if ( n >= 6 && buf[n - 2] == ESC[0] && buf[n - 1] == '\\' )
       buf[n - 2] = '\0';
 
-    color_str = buf.data();
+    return buf.data();
   }
 
-  return color_str;
+  return {};
 }
 
 //----------------------------------------------------------------------
-FString FTermDetection::parseAnswerbackMsg (const FString& current_termtype)
+auto FTermDetection::parseAnswerbackMsg (const FString& current_termtype) -> FString
 {
   FString new_termtype{current_termtype};
   static auto& keyboard = FKeyboard::getInstance();
@@ -590,47 +573,22 @@ FString FTermDetection::parseAnswerbackMsg (const FString& current_termtype)
 }
 
 //----------------------------------------------------------------------
-FString FTermDetection::getAnswerbackMsg() const
+auto FTermDetection::getAnswerbackMsg() const -> FString
 {
-  FString answerback{""};
-  fd_set ifds{};
-  struct timeval tv{};
-  const int stdin_no = FTermios::getStdIn();
   // Send enquiry character
   std::putchar (ENQ[0]);
   std::fflush(stdout);
-  FD_ZERO(&ifds);
-  FD_SET(stdin_no, &ifds);
-  tv.tv_sec  = 0;
-  tv.tv_usec = 150000;  // 150 ms
-
-  // Read the answerback message
-  if ( select (stdin_no + 1, &ifds, nullptr, nullptr, &tv) < 1 )
-    return answerback;
-
   std::array<char, 10> temp{};
-  std::size_t pos{0};
-
-  do
-  {
-    std::size_t bytes_free = temp.size() - pos - 1;
-    const ssize_t bytes = read(stdin_no, &temp[pos], bytes_free);
-
-    if ( bytes <= 0 )
-      break;
-
-    pos += std::size_t(bytes);
-  }
-  while ( pos < temp.size() );
+  auto pos = captureTerminalInput(temp, 150'000);
 
   if ( pos > 0 )
-    answerback = temp.data();
+    return temp.data();
 
-  return answerback;
+  return {};
 }
 
 //----------------------------------------------------------------------
-FString FTermDetection::parseSecDA (const FString& current_termtype)
+auto FTermDetection::parseSecDA (const FString& current_termtype) -> FString
 {
   // The Linux console and older cygwin terminals knows no Sec_DA
 
@@ -650,8 +608,8 @@ FString FTermDetection::parseSecDA (const FString& current_termtype)
   // remove the last byte ("c")
   temp.remove(temp.getLength() - 1, 1);
   // split into components
-  const auto sec_da_components = temp.split(';');
-  const auto num_components = sec_da_components.size();
+  const auto& sec_da_components = temp.split(';');
+  const auto& num_components = sec_da_components.size();
 
   // The second device attribute (SEC_DA) always has 3 parameters,
   // otherwise it usually has a copy of the device attribute (primary DA)
@@ -670,7 +628,7 @@ FString FTermDetection::parseSecDA (const FString& current_termtype)
   // Read the terminal hardware option
   secondary_da.terminal_id_hardware = str2int(sec_da_components[2]);
 
-  FString new_termtype = secDA_Analysis(current_termtype);
+  const auto& new_termtype = secDA_Analysis(current_termtype);
 
 #if DEBUG
   if ( ! new_termtype.isEmpty() )
@@ -681,7 +639,7 @@ FString FTermDetection::parseSecDA (const FString& current_termtype)
 }
 
 //----------------------------------------------------------------------
-int FTermDetection::str2int (const FString& s) const
+auto FTermDetection::str2int (const FString& s) const -> int
 {
   // This is not a general string to integer conversion method.
   // It is only used in this class to convert the device attribute
@@ -711,48 +669,24 @@ int FTermDetection::str2int (const FString& s) const
 }
 
 //----------------------------------------------------------------------
-FString FTermDetection::getSecDA() const
+auto FTermDetection::getSecDA() const -> FString
 {
-  FString sec_da_str{""};
-
-  int a{0};
-  int b{0};
-  int c{0};
-  const int stdin_no{FTermios::getStdIn()};
-  const int stdout_no{FTermios::getStdOut()};
-  fd_set ifds{};
-  struct timeval tv{};
+  const auto& stdout_no{FTermios::getStdOut()};
   const std::string SECDA{ESC "[>c"};
 
   // Get the secondary device attributes
   if ( write(stdout_no, SECDA.data(), SECDA.length()) == -1 )
-    return sec_da_str;
+    return {};
 
   std::fflush(stdout);
-  FD_ZERO(&ifds);
-  FD_SET(stdin_no, &ifds);
-  tv.tv_sec  = 0;
-  tv.tv_usec = 600000;  // 600 ms
-
-  // Read the answer
-  if ( select (stdin_no + 1, &ifds, nullptr, nullptr, &tv) < 1 )
-    return sec_da_str;
-
-  constexpr auto parse = "\033[>%10d;%10d;%10dc";
   std::array<char, 40> temp{};
-  std::size_t pos{0};
-
-  do
-  {
-    std::size_t bytes_free = temp.size() - pos - 1;
-    const ssize_t bytes = read(stdin_no, &temp[pos], bytes_free);
-
-    if ( bytes <= 0 )
-      break;
-
-    pos += std::size_t(bytes);
-  }
-  while ( pos < temp.size() && ! std::strchr(temp.data(), 'c') );
+  auto isWithout_c = [] (const auto& t) { return ! std::strchr(t.data(), 'c'); };
+  auto pos = captureTerminalInput(temp, 600'000, isWithout_c);
+  constexpr auto parse = "\033[>%10d;%10d;%10dc";
+  FString sec_da_str{""};
+  int a{0};
+  int b{0};
+  int c{0};
 
   if ( pos > 3 && std::sscanf(temp.data(), parse, &a, &b, &c) == 3 )
     sec_da_str.sprintf("\033[>%d;%d;%dc", a, b, c);
@@ -761,7 +695,7 @@ FString FTermDetection::getSecDA() const
 }
 
 //----------------------------------------------------------------------
-FString FTermDetection::secDA_Analysis (const FString& current_termtype)
+auto FTermDetection::secDA_Analysis (const FString& current_termtype) -> FString
 {
   FString new_termtype{current_termtype};
 
@@ -840,7 +774,7 @@ FString FTermDetection::secDA_Analysis (const FString& current_termtype)
 }
 
 //----------------------------------------------------------------------
-inline FString FTermDetection::secDA_Analysis_0 (const FString& current_termtype) const
+inline auto FTermDetection::secDA_Analysis_0 (const FString& current_termtype) const -> FString
 {
   // Terminal ID 0 - DEC VT100
 
@@ -864,7 +798,7 @@ inline FString FTermDetection::secDA_Analysis_0 (const FString& current_termtype
 }
 
 //----------------------------------------------------------------------
-inline FString FTermDetection::secDA_Analysis_1 (const FString& current_termtype)
+inline auto FTermDetection::secDA_Analysis_1 (const FString& current_termtype) -> FString
 {
   // Terminal ID 1 - DEC VT220
 
@@ -880,7 +814,7 @@ inline FString FTermDetection::secDA_Analysis_1 (const FString& current_termtype
 }
 
 //----------------------------------------------------------------------
-inline FString FTermDetection::secDA_Analysis_24 (const FString& current_termtype)
+inline auto FTermDetection::secDA_Analysis_24 (const FString& current_termtype) -> FString
 {
   // Terminal ID 24 - DEC VT320
 
@@ -909,7 +843,7 @@ inline FString FTermDetection::secDA_Analysis_24 (const FString& current_termtyp
 }
 
 //----------------------------------------------------------------------
-inline FString FTermDetection::secDA_Analysis_32() const
+inline auto FTermDetection::secDA_Analysis_32() const -> FString
 {
   // Terminal ID 32 - Tera Term
 
@@ -919,7 +853,7 @@ inline FString FTermDetection::secDA_Analysis_32() const
 }
 
 //----------------------------------------------------------------------
-inline FString FTermDetection::secDA_Analysis_65 (const FString& current_termtype)
+inline auto FTermDetection::secDA_Analysis_65 (const FString& current_termtype) -> FString
 {
   // Terminal ID 65 - DEC VT525 and VTE >= 0.53.0
 
@@ -927,7 +861,7 @@ inline FString FTermDetection::secDA_Analysis_65 (const FString& current_termtyp
 }
 
 //----------------------------------------------------------------------
-inline FString FTermDetection::secDA_Analysis_67() const
+inline auto FTermDetection::secDA_Analysis_67() const -> FString
 {
   // Terminal ID 67 - cygwin
 
@@ -938,7 +872,7 @@ inline FString FTermDetection::secDA_Analysis_67() const
 }
 
 //----------------------------------------------------------------------
-inline FString FTermDetection::secDA_Analysis_77()
+inline auto FTermDetection::secDA_Analysis_77() -> FString
 {
   // Terminal ID 77 - mintty
 
@@ -950,7 +884,7 @@ inline FString FTermDetection::secDA_Analysis_77()
 }
 
 //----------------------------------------------------------------------
-inline FString FTermDetection::secDA_Analysis_82() const
+inline auto FTermDetection::secDA_Analysis_82() const -> FString
 {
   // Terminal ID 82 - rxvt
 
@@ -967,7 +901,7 @@ inline FString FTermDetection::secDA_Analysis_82() const
 }
 
 //----------------------------------------------------------------------
-inline FString FTermDetection::secDA_Analysis_83 (const FString& current_termtype) const
+inline auto FTermDetection::secDA_Analysis_83 (const FString& current_termtype) const -> FString
 {
   // Terminal ID 83 - screen
 
@@ -977,7 +911,7 @@ inline FString FTermDetection::secDA_Analysis_83 (const FString& current_termtyp
 }
 
 //----------------------------------------------------------------------
-inline FString FTermDetection::secDA_Analysis_84 (const FString& current_termtype) const
+inline auto FTermDetection::secDA_Analysis_84 (const FString& current_termtype) const -> FString
 {
   // Terminal ID 84 - tmux
 
@@ -988,7 +922,7 @@ inline FString FTermDetection::secDA_Analysis_84 (const FString& current_termtyp
 }
 
 //----------------------------------------------------------------------
-inline FString FTermDetection::secDA_Analysis_85() const
+inline auto FTermDetection::secDA_Analysis_85() const -> FString
 {
   // Terminal ID 85 - rxvt-unicode
 
@@ -1011,7 +945,7 @@ inline FString FTermDetection::secDA_Analysis_85() const
 }
 
 //----------------------------------------------------------------------
-inline FString FTermDetection::secDA_Analysis_vte (const FString& current_termtype)
+inline auto FTermDetection::secDA_Analysis_vte (const FString& current_termtype) -> FString
 {
   // VTE terminal library
   // (Since VTE 0.53.0 the terminal ID has changed from 1 to 65)
@@ -1025,7 +959,7 @@ inline FString FTermDetection::secDA_Analysis_vte (const FString& current_termty
     // Each gnome-terminal should be able to use 256 colors
     color256 = true;
     new_termtype = "gnome-256color";
-    auto id = secondary_da.terminal_id_version;
+    const auto& id = secondary_da.terminal_id_version;
     fterm_data.setGnomeTerminalID(id);
 
     // VTE 0.40.0 or higher and gnome-terminal 3.16 or higher
@@ -1037,7 +971,7 @@ inline FString FTermDetection::secDA_Analysis_vte (const FString& current_termty
 }
 
 //----------------------------------------------------------------------
-inline FString FTermDetection::secDA_Analysis_kitty (const FString& current_termtype)
+inline auto FTermDetection::secDA_Analysis_kitty (const FString& current_termtype) -> FString
 {
   // kitty
 
@@ -1048,8 +982,8 @@ inline FString FTermDetection::secDA_Analysis_kitty (const FString& current_term
     // All kitty terminals can use 256 colors
     color256 = true;
     new_termtype = "xterm-kitty";
-    int n1 = secondary_da.terminal_id_version - 4000;
-    int n2 = secondary_da.terminal_id_hardware;
+    const auto& n1 = secondary_da.terminal_id_version - 4000;
+    const auto& n2 = secondary_da.terminal_id_hardware;
     FTermData::kittyVersion kitty_version { n1, n2 };
     FTermData::getInstance().setKittyVersion(kitty_version);
   }
