@@ -3,7 +3,7 @@
 *                                                                      *
 * This file is part of the FINAL CUT widget toolkit                    *
 *                                                                      *
-* Copyright 2012-2021 Markus Gans                                      *
+* Copyright 2012-2023 Markus Gans                                      *
 *                                                                      *
 * FINAL CUT is free software; you can redistribute it and/or modify    *
 * it under the terms of the GNU Lesser General Public License as       *
@@ -134,13 +134,13 @@ void FScrollbar::setOrientation (Orientation o)
 
   if ( o == Orientation::Vertical && bar_orientation == Orientation::Horizontal )
   {
-    setWidth(1);
-    setHeight(length);
+    FWidget::setWidth(1);
+    FWidget::setHeight(length);
   }
   else if ( o == Orientation::Horizontal && bar_orientation == Orientation::Vertical )
   {
-    setWidth(length);
-    setHeight(1);
+    FWidget::setWidth(length);
+    FWidget::setHeight(1);
   }
 
   calculateSliderValues();
@@ -247,7 +247,10 @@ void FScrollbar::onMouseDown (FMouseEvent* ev)
     && ev->getButton() != MouseButton::Middle )
     return;
 
-  setWidgetFocus(getParentWidget());
+  const auto& parent_widget = getParentWidget();
+
+  if ( parent_widget && ! parent_widget->isInstanceOf("FScrollView") )
+    setWidgetFocus(parent_widget);
 
   if ( min == max )
     return;
@@ -355,7 +358,6 @@ void FScrollbar::onMouseMove (FMouseEvent* ev)
     {
       setValue(new_val);
       drawBar();
-      forceTerminalUpdate();
       processScroll();
     }
   }
@@ -616,7 +618,7 @@ void FScrollbar::drawButtons()
 }
 
 //----------------------------------------------------------------------
-FScrollbar::ScrollType FScrollbar::getClickedScrollType (int x, int y) const
+auto FScrollbar::getClickedScrollType (int x, int y) const -> ScrollType
 {
   if ( bar_orientation == Orientation::Vertical )
     return getVerticalClickedScrollType(y);
@@ -625,7 +627,7 @@ FScrollbar::ScrollType FScrollbar::getClickedScrollType (int x, int y) const
 }
 
 //----------------------------------------------------------------------
-FScrollbar::ScrollType FScrollbar::getVerticalClickedScrollType (int y) const
+auto FScrollbar::getVerticalClickedScrollType (int y) const -> ScrollType
 {
   if ( y == 1 )
     return ScrollType::StepBackward;  // decrement button
@@ -643,7 +645,7 @@ FScrollbar::ScrollType FScrollbar::getVerticalClickedScrollType (int y) const
 }
 
 //----------------------------------------------------------------------
-FScrollbar::ScrollType FScrollbar::getHorizontalClickedScrollType (int x) const
+auto FScrollbar::getHorizontalClickedScrollType (int x) const -> ScrollType
 {
   if ( FVTerm::getFOutput()->isNewFont() )
   {
@@ -678,7 +680,7 @@ FScrollbar::ScrollType FScrollbar::getHorizontalClickedScrollType (int x) const
 }
 
 //----------------------------------------------------------------------
-int FScrollbar::getSliderClickPos (int mouse_x, int mouse_y) const
+auto FScrollbar::getSliderClickPos (int mouse_x, int mouse_y) const -> int
 {
   // Get the clicked position on the slider
 
@@ -739,7 +741,6 @@ void FScrollbar::jumpToClickPos (int x, int y)
   {
     setValue(new_val);
     drawBar();
-    forceTerminalUpdate();
     scroll_type = ScrollType::Jump;
     processScroll();
   }
