@@ -4,7 +4,7 @@
 *                                                                      *
 * This file is part of the FINAL CUT widget toolkit                    *
 *                                                                      *
-* Copyright 2017-2021 Markus Gans                                      *
+* Copyright 2017-2023 Markus Gans                                      *
 *                                                                      *
 * FINAL CUT is free software; you can redistribute it and/or modify    *
 * it under the terms of the GNU Lesser General Public License as       *
@@ -21,11 +21,14 @@
 * <http://www.gnu.org/licenses/>.                                      *
 ***********************************************************************/
 
+#include <algorithm>
 #include <memory>
 
 #include "final/fevent.h"
 #include "final/fwidgetcolors.h"
+#include "final/vterm/fcolorpair.h"
 #include "final/widget/fscrollview.h"
+#include "final/widget/fstatusbar.h"
 #include "final/widget/fwindow.h"
 
 namespace finalcut
@@ -46,9 +49,7 @@ FScrollView::FScrollView (FWidget* parent)
 //----------------------------------------------------------------------
 FScrollView::~FScrollView()  // destructor
 {
-  removeArea (viewport);
-  viewport = nullptr;
-  setChildPrintArea (viewport);
+  setChildPrintArea (viewport.get());
 }
 
 
@@ -64,15 +65,15 @@ void FScrollView::setScrollWidth (std::size_t width)
 
   if ( viewport )
   {
-    const FSize no_shadow(0, 0);
     scroll_geometry.setWidth (width);
-    resizeArea (scroll_geometry, no_shadow, viewport);
-
+    resizeArea (scroll_geometry, viewport.get());
+    setColor();
+    clearArea();
     addPreprocessingHandler
     (
       F_PREPROC_HANDLER (this, &FScrollView::copy2area)
     );
-    setChildPrintArea (viewport);
+    setChildPrintArea (viewport.get());
   }
 
   hbar->setMaximum (int(width - getViewportWidth()));
@@ -94,14 +95,15 @@ void FScrollView::setScrollHeight (std::size_t height)
 
   if ( viewport )
   {
-    const FSize no_shadow(0, 0);
     scroll_geometry.setHeight (height);
-    resizeArea (scroll_geometry, no_shadow, viewport);
+    resizeArea (scroll_geometry, viewport.get());
+    setColor();
+    clearArea();
     addPreprocessingHandler
     (
       F_PREPROC_HANDLER (this, &FScrollView::copy2area)
     );
-    setChildPrintArea (viewport);
+    setChildPrintArea (viewport.get());
   }
 
   vbar->setMaximum (int(height - getViewportHeight()));
@@ -115,28 +117,23 @@ void FScrollView::setScrollHeight (std::size_t height)
 //----------------------------------------------------------------------
 void FScrollView::setScrollSize (const FSize& size)
 {
-  std::size_t width = size.getWidth();
-  std::size_t height = size.getHeight();
-
-  if ( width < getViewportWidth() )
-    width = getViewportWidth();
-
-  if ( height < getViewportHeight() )
-    height = getViewportHeight();
+  std::size_t width = std::max(size.getWidth(), getViewportWidth());
+  std::size_t height = std::max(size.getHeight(), getViewportHeight());
 
   if ( getScrollWidth() == width && getScrollHeight() == height )
     return;
 
   if ( viewport )
   {
-    const FSize no_shadow(0, 0);
     scroll_geometry.setSize (width, height);
-    resizeArea (scroll_geometry, no_shadow, viewport);
+    resizeArea (scroll_geometry, viewport.get());
+    setColor();
+    clearArea();
     addPreprocessingHandler
     (
       F_PREPROC_HANDLER (this, &FScrollView::copy2area)
     );
-    setChildPrintArea (viewport);
+    setChildPrintArea (viewport.get());
   }
 
   const auto xoffset_end = int(getScrollWidth() - getViewportWidth());
@@ -166,15 +163,15 @@ void FScrollView::setX (int x, bool adjust)
 {
   FWidget::setX (x, adjust);
 
-  if ( ! adjust )
-  {
-    scroll_geometry.setX (getTermX() + getLeftPadding() - 1);
+  if ( adjust )
+    return;
 
-    if ( viewport )
-    {
-      viewport->offset_left = scroll_geometry.getX();
-      viewport->offset_top = scroll_geometry.getY();
-    }
+  scroll_geometry.setX (getTermX() + getLeftPadding() - 1);
+
+  if ( viewport )
+  {
+    viewport->offset_left = scroll_geometry.getX();
+    viewport->offset_top = scroll_geometry.getY();
   }
 }
 
@@ -183,15 +180,15 @@ void FScrollView::setY (int y, bool adjust)
 {
   FWidget::setY (y, adjust);
 
-  if ( ! adjust )
-  {
-    scroll_geometry.setY (getTermY() + getTopPadding() - 1);
+  if ( adjust )
+    return;
 
-    if ( viewport )
-    {
-      viewport->offset_left = scroll_geometry.getX();
-      viewport->offset_top = scroll_geometry.getY();
-    }
+  scroll_geometry.setY (getTermY() + getTopPadding() - 1);
+
+  if ( viewport )
+  {
+    viewport->offset_left = scroll_geometry.getX();
+    viewport->offset_top = scroll_geometry.getY();
   }
 }
 
@@ -202,11 +199,11 @@ void FScrollView::setPos (const FPoint& p, bool adjust)
   scroll_geometry.setPos ( getTermX() + getLeftPadding() - 1
                          , getTermY() + getTopPadding() - 1 );
 
-  if ( ! adjust && viewport )
-  {
-    viewport->offset_left = scroll_geometry.getX();
-    viewport->offset_top = scroll_geometry.getY();
-  }
+  if ( adjust || ! viewport )
+    return;
+
+  viewport->offset_left = scroll_geometry.getX();
+  viewport->offset_top = scroll_geometry.getY();
 }
 
 //----------------------------------------------------------------------
@@ -221,6 +218,14 @@ void FScrollView::setWidth (std::size_t w, bool adjust)
 
   if ( getScrollWidth() < getViewportWidth() )
     setScrollWidth (getViewportWidth());
+
+  if ( ! viewport )
+    return;
+
+  // Insufficient space scrolling
+  viewport_geometry.x1_ref() = \
+      std::min ( int(getScrollWidth() - getViewportWidth())
+               , viewport_geometry.getX1() );
 }
 
 //----------------------------------------------------------------------
@@ -235,26 +240,27 @@ void FScrollView::setHeight (std::size_t h, bool adjust)
 
   if ( getScrollHeight() < getViewportHeight() )
     setScrollHeight (getViewportHeight());
+
+  if ( ! viewport )
+    return;
+
+  // Insufficient space scrolling
+  viewport_geometry.y1_ref() = \
+      std::min ( int(getScrollHeight() - getViewportHeight())
+               , viewport_geometry.getY1() );
 }
 
 //----------------------------------------------------------------------
 void FScrollView::setSize (const FSize& size, bool adjust)
 {
-  const std::size_t w = size.getWidth();
-  const std::size_t h = size.getHeight();
+  // Set the scroll view size
 
-  if ( w <= vertical_border_spacing + nf_offset
-    || h <= horizontal_border_spacing )
-    return;
+   if ( size.getWidth() <= vertical_border_spacing + nf_offset
+     || size.getHeight() <= horizontal_border_spacing )
+     return;
 
   FWidget::setSize (size, adjust);
-  viewport_geometry.setSize ( w - vertical_border_spacing - nf_offset
-                            , h - horizontal_border_spacing );
-  calculateScrollbarPos();
-
-  if ( getScrollWidth() < getViewportWidth()
-    || getScrollHeight() < getViewportHeight() )
-    setScrollSize (getViewportSize());
+  changeSize (size, adjust);
 }
 
 //----------------------------------------------------------------------
@@ -264,28 +270,13 @@ void FScrollView::setGeometry ( const FPoint& pos, const FSize& size
   // Set the scroll view geometry
 
   FWidget::setGeometry (pos, size, adjust);
-  const std::size_t w = size.getWidth();
-  const std::size_t h = size.getHeight();
   scroll_geometry.setPos ( getTermX() + getLeftPadding() - 1
                          , getTermY() + getTopPadding() - 1 );
-  viewport_geometry.setSize ( w - vertical_border_spacing - nf_offset
-                            , h - horizontal_border_spacing );
-  calculateScrollbarPos();
-
-  if ( getScrollWidth() < getViewportWidth()
-    || getScrollHeight() < getViewportHeight() )
-  {
-    FScrollView::setScrollSize (getViewportSize());
-  }
-  else if ( ! adjust && viewport )
-  {
-    viewport->offset_left = scroll_geometry.getX();
-    viewport->offset_top = scroll_geometry.getY();
-  }
+  changeSize (size, adjust);
 }
 
 //----------------------------------------------------------------------
-bool FScrollView::setCursorPos (const FPoint& p)
+auto FScrollView::setCursorPos (const FPoint& p) -> bool
 {
   return FWidget::setCursorPos ({ p.getX() + getLeftPadding()
                                 , p.getY() + getTopPadding() });
@@ -299,7 +290,19 @@ void FScrollView::setPrintPos (const FPoint& p)
 }
 
 //----------------------------------------------------------------------
-bool FScrollView::setViewportPrint (bool enable)
+void FScrollView::setText (const FString& txt)
+{
+  text.setString(txt);
+
+  if ( isEnabled() )
+  {
+    delAccelerator();
+    setHotkeyAccelerator();
+  }
+}
+
+//----------------------------------------------------------------------
+auto FScrollView::setViewportPrint (bool enable) -> bool
 {
   return (use_own_print_area = ! enable);
 }
@@ -314,16 +317,9 @@ void FScrollView::resetColors()
 }
 
 //----------------------------------------------------------------------
-bool FScrollView::setBorder (bool enable)
+auto FScrollView::setBorder (bool enable) -> bool
 {
-  return (setFlags().no_border = ! enable);
-}
-
-//----------------------------------------------------------------------
-bool FScrollView::setFocus (bool)
-{
-  // This container widget cannot have its own focus
-  return false;
+  return (setFlags().feature.no_border = ! enable);
 }
 
 //----------------------------------------------------------------------
@@ -348,7 +344,7 @@ void FScrollView::setVerticalScrollBarMode (ScrollBarMode mode)
 void FScrollView::clearArea (wchar_t fillchar)
 {
   if ( viewport )
-    clearArea (viewport, fillchar);
+    clearArea (viewport.get(), fillchar);
 }
 
 //----------------------------------------------------------------------
@@ -380,21 +376,10 @@ void FScrollView::scrollTo (int x, int y)
   if ( xoffset == x && yoffset == y )
     return;
 
-  xoffset = x;
-  yoffset = y;
-
-  if ( yoffset < 0 )
-    yoffset = 0;
-
-  if ( yoffset > yoffset_end )
-    yoffset = yoffset_end;
-
-  if ( xoffset < 0 )
-    xoffset = 0;
-
-  if ( xoffset > xoffset_end )
-    xoffset = xoffset_end;
-
+  xoffset = std::max(x, 0);
+  xoffset = std::min(xoffset, xoffset_end);
+  yoffset = std::max(y, 0);
+  yoffset = std::min(yoffset, yoffset_end);
   const bool changeX( xoffset_before != xoffset );
   const bool changeY( yoffset_before != yoffset );
 
@@ -429,7 +414,6 @@ void FScrollView::scrollTo (int x, int y)
 
   viewport->has_changes = true;
   copy2area();
-  forceTerminalUpdate();
 }
 
 //----------------------------------------------------------------------
@@ -468,6 +452,7 @@ void FScrollView::draw()
 
   vbar->redraw();
   hbar->redraw();
+  drawLabel();
 }
 
 //----------------------------------------------------------------------
@@ -477,18 +462,71 @@ void FScrollView::drawBorder()
   finalcut::drawListBorder (this, box);
 }
 
+//----------------------------------------------------------------------
+void FScrollView::drawLabel()
+{
+  if ( text.isEmpty() )
+    return;
+
+  FString label_text{};
+  const FString txt{" " + text + " "};
+  unsetViewportPrint();
+  const auto hotkeypos = finalcut::getHotkeyPos(txt, label_text);
+
+  if ( hasBorder() )
+    FWidget::setPrintPos (FPoint{2, 1});
+  else
+    FWidget::setPrintPos (FPoint{0, 1});
+
+  drawText (label_text, hotkeypos);
+  setViewportPrint();
+}
+
 //----------------------------------------------------------------------
 void FScrollView::onKeyPress (FKeyEvent* ev)
 {
-  const auto& entry = key_map[ev->key()];
+  const auto& iter = key_map.find(ev->key());
 
-  if ( entry )
+  if ( iter != key_map.end() )
   {
-    entry();
+    iter->second();
     ev->accept();
   }
 }
 
+//----------------------------------------------------------------------
+void FScrollView::onMouseDown (FMouseEvent* ev)
+{
+  if ( ev->getButton() != MouseButton::Left )
+    return;
+
+  const int mouse_x = ev->getX();
+  const int mouse_y = ev->getY();
+
+  if ( mouse_x == 1 || mouse_x == int(getWidth())
+    || mouse_y == 1 || mouse_y == int(getHeight()) )
+  {
+    directFocus();
+  }
+
+  // Event handover to parent dialog
+  passResizeCornerEventToDialog(this, *ev);
+}
+
+//----------------------------------------------------------------------
+void FScrollView::onMouseUp (FMouseEvent* ev)
+{
+  // Event handover to parent dialog
+  passResizeCornerEventToDialog(this, *ev);
+}
+
+//----------------------------------------------------------------------
+void FScrollView::onMouseMove (FMouseEvent* ev)
+{
+  // Event handover to parent dialog
+  passResizeCornerEventToDialog(this, *ev);
+}
+
 //----------------------------------------------------------------------
 void FScrollView::onWheel (FWheelEvent* ev)
 {
@@ -504,28 +542,27 @@ void FScrollView::onWheel (FWheelEvent* ev)
   }
 }
 
+//----------------------------------------------------------------------
+void FScrollView::onAccel (FAccelEvent*)
+{
+  directFocus();
+}
+
 //----------------------------------------------------------------------
 void FScrollView::onFocusIn (FFocusEvent* in_ev)
 {
   // Sets the focus to a child widget if it exists
 
-  if ( hasChildren() )
+  if ( ! hasChildren() )
+    FWidget::onFocusIn(in_ev);
+
+  if ( in_ev->getFocusType() == FocusTypes::NextWidget )
   {
-    auto prev_element = getFocusWidget();
-
-    if ( in_ev->getFocusType() == FocusTypes::NextWidget )
-      focusFirstChild();
-    else if ( in_ev->getFocusType() == FocusTypes::PreviousWidget )
-      focusLastChild();
-
-    if ( prev_element )
-      prev_element->redraw();
-
-    if ( getFocusWidget() )
-      getFocusWidget()->redraw();
-
-    FFocusEvent cfi (Event::ChildFocusIn);
-    onChildFocusIn(&cfi);
+    focusFirstChild() ? in_ev->accept() : in_ev->ignore();
+  }
+  else if ( in_ev->getFocusType() == FocusTypes::PreviousWidget )
+  {
+    focusLastChild() ? in_ev->accept() : in_ev->ignore();
   }
 }
 
@@ -539,29 +576,20 @@ void FScrollView::onChildFocusIn (FFocusEvent*)
   if ( ! focus )
     return;
 
-  const FRect widget_geometry = focus->getGeometryWithShadow();
+  const auto& widget_geometry = focus->getGeometryWithShadow();
   FRect vp_geometry = viewport_geometry;
   vp_geometry.move(1, 1);
 
   if ( ! vp_geometry.contains(widget_geometry) )
   {
-    int x{};
-    int y{};
     const int vx = vp_geometry.getX();
     const int vy = vp_geometry.getY();
     const int wx = widget_geometry.getX();
     const int wy = widget_geometry.getY();
-
-    if ( wx > vx )
-      x = widget_geometry.getX2() - int(vp_geometry.getWidth()) + 1;
-    else
-      x = wx;
-
-    if ( wy > vy )
-      y = widget_geometry.getY2() - int(vp_geometry.getHeight()) + 1;
-    else
-      y = wy;
-
+    const auto width = int(vp_geometry.getWidth());
+    const auto height = int(vp_geometry.getHeight());
+    const int x = ( wx > vx ) ? widget_geometry.getX2() - width + 1 : wx;
+    const int y = ( wy > vy ) ? widget_geometry.getY2() - height + 1 : wy;
     scrollTo (x, y);
   }
 }
@@ -577,28 +605,45 @@ void FScrollView::onChildFocusOut (FFocusEvent* out_ev)
   {
     const auto& last_widget = getLastFocusableWidget(getChildren());
 
-    if ( focus == last_widget )
-    {
-      out_ev->accept();
-      focusNextChild();
-    }
+    if ( focus != last_widget )
+      return;
+
+    out_ev->accept();
+    focusNextChild();
   }
   else if ( out_ev->getFocusType() == FocusTypes::PreviousWidget )
   {
     const auto& first_widget = getFirstFocusableWidget(getChildren());
 
-    if ( focus == first_widget )
-    {
-      out_ev->accept();
-      focusPrevChild();
-    }
+    if ( focus != first_widget )
+      return;
+
+    out_ev->accept();
+    focusPrevChild();
+  }
+}
+
+//----------------------------------------------------------------------
+void FScrollView::onFailAtChildFocus (FFocusEvent* fail_ev)
+{
+  // Change the focus away from FScrollView to another widget
+
+  if ( fail_ev->getFocusType() == FocusTypes::NextWidget )
+  {
+    fail_ev->accept();
+    focusNextChild();
+  }
+  else if ( fail_ev->getFocusType() == FocusTypes::PreviousWidget )
+  {
+    fail_ev->accept();
+    focusPrevChild();
   }
 }
 
 
 // protected methods of FScrollView
 //----------------------------------------------------------------------
-FVTerm::FTermArea* FScrollView::getPrintArea()
+auto FScrollView::getPrintArea() -> FVTerm::FTermArea*
 {
   // returns print area or viewport
 
@@ -606,11 +651,17 @@ FVTerm::FTermArea* FScrollView::getPrintArea()
   {
     setChildPrintArea (nullptr);
     auto area = FWidget::getPrintArea();
-    setChildPrintArea (viewport);
+    setChildPrintArea (viewport.get());
     return area;
   }
 
-  return viewport;
+  return viewport.get();
+}
+
+//----------------------------------------------------------------------
+void FScrollView::setHotkeyAccelerator()
+{
+  setHotkeyViaString (this, text);
 }
 
 //----------------------------------------------------------------------
@@ -690,19 +741,14 @@ void FScrollView::copy2area()
 
   for (auto y{0}; y < y_end; y++)  // line loop
   {
-    const int v_line_len = viewport->width;
-    const int a_line_len = printarea->width + printarea->right_shadow;
     // viewport character
-    const auto& vc = viewport->data[(dy + y) * v_line_len + dx];
+    const auto& vc = viewport->getFChar(dx, dy + y);
     // area character
-    auto& ac = printarea->data[(ay + y) * a_line_len + ax];
+    auto& ac = printarea->getFChar(ax, ay + y);
     std::memcpy (&ac, &vc, sizeof(FChar) * unsigned(x_end));
-
-    if ( int(printarea->changes[ay + y].xmin) > ax )
-      printarea->changes[ay + y].xmin = uInt(ax);
-
-    if ( int(printarea->changes[ay + y].xmax) < ax + x_end - 1 )
-      printarea->changes[ay + y].xmax = uInt(ax + x_end - 1);
+    auto& line_changes = printarea->changes[std::size_t(ay + y)];
+    line_changes.xmin = std::min(line_changes.xmin, uInt(ax));
+    line_changes.xmax = std::max(line_changes.xmax, uInt(ax + x_end - 1));
   }
 
   setViewportCursor();
@@ -713,7 +759,7 @@ void FScrollView::copy2area()
 
 // private methods of FScrollView
 //----------------------------------------------------------------------
-inline FPoint FScrollView::getViewportCursorPos()
+inline auto FScrollView::getViewportCursorPos() -> FPoint
 {
   auto window = FWindow::getWindowWidget(this);
 
@@ -734,7 +780,7 @@ inline FPoint FScrollView::getViewportCursorPos()
 //----------------------------------------------------------------------
 void FScrollView::init()
 {
-  const FWidget* parent = getParentWidget();
+  const auto& parent = getParentWidget();
 
   assert ( parent != nullptr );
   assert ( ! parent->isInstanceOf("FScrollView") );
@@ -745,43 +791,150 @@ void FScrollView::init()
   FScrollView::resetColors();
   FScrollView::setGeometry (FPoint{1, 1}, FSize{4, 4});
   setMinimumSize (FSize{4, 4});
-  const FSize no_shadow{0, 0};
-  std::size_t w = getViewportWidth();
-  std::size_t h = getViewportHeight();
-
-  if ( w < 1 )
-    w = 1;
-
-  if ( h < 1 )
-    h = 1;
-
-  scroll_geometry.setRect (0, 0, w, h);
-  createArea (scroll_geometry, no_shadow, viewport);
+  std::size_t width = std::max(getViewportWidth(), std::size_t(1));
+  std::size_t height = std::max(getViewportHeight(), std::size_t(1));
+  createViewport({width, height});
   addPreprocessingHandler
   (
     F_PREPROC_HANDLER (this, &FScrollView::copy2area)
   );
 
   if ( viewport )
-    setChildPrintArea (viewport);
+    setChildPrintArea (viewport.get());
+}
+
+//----------------------------------------------------------------------
+inline void FScrollView::createViewport (const FSize& size) noexcept
+{
+  // Initialization of the scrollable viewport
+
+  scroll_geometry.setSize(size);
+  viewport = createArea(scroll_geometry);
+  setColor();
+  clearArea();
+}
+
+//----------------------------------------------------------------------
+void FScrollView::drawText ( const FString& label_text
+                           , std::size_t hotkeypos )
+{
+  const auto& wc = getColorTheme();
+  const std::size_t column_width = getColumnWidth(label_text);
+  std::size_t length = label_text.getLength();
+  bool ellipsis{false};
+
+  if ( column_width > getClientWidth() )
+  {
+    const std::size_t len = getClientWidth() - 3;
+    const FString s = finalcut::getColumnSubString (label_text, 1, len);
+    length = s.getLength();
+    ellipsis = true;
+  }
+
+  if ( FVTerm::getFOutput()->isMonochron() )
+    setReverse(true);
+
+  if ( isEnabled() )
+    setColor(wc->label_emphasis_fg, wc->label_bg);
+  else
+    setColor(wc->label_inactive_fg, wc->label_inactive_bg);
+
+  for (std::size_t z{0}; z < length; z++)
+  {
+    if ( (z == hotkeypos) && getFlags().feature.active )
+    {
+      setColor (wc->label_hotkey_fg, wc->label_hotkey_bg);
+
+      if ( ! getFlags().feature.no_underline )
+        setUnderline();
+
+      print (label_text[z]);
+
+      if ( ! getFlags().feature.no_underline )
+        unsetUnderline();
+
+      setColor (wc->label_emphasis_fg, wc->label_bg);
+    }
+    else
+      print (label_text[z]);
+  }
+
+  if ( ellipsis )  // Print ellipsis
+    print() << FColorPair {wc->label_ellipsis_fg, wc->label_bg} << "..";
+
+  if ( FVTerm::getFOutput()->isMonochron() )
+    setReverse(true);
+}
+
+//----------------------------------------------------------------------
+void FScrollView::directFocus()
+{
+  auto focused_widget = getFocusWidget();
+  focusFirstChild();
+
+  if ( focused_widget )
+    focused_widget->redraw();
+
+  focused_widget = getFocusWidget();
+
+  if ( focused_widget )
+    focused_widget->redraw();
+
+  drawStatusBarMessage();
 }
 
 //----------------------------------------------------------------------
 inline void FScrollView::mapKeyFunctions()
 {
-  key_map[FKey::Up]        = [this] { scrollBy (0, -1); };
-  key_map[FKey::Down]      = [this] { scrollBy (0, 1); };
-  key_map[FKey::Left]      = [this] { scrollBy (-1, 0); };
-  key_map[FKey::Right]     = [this] { scrollBy (1, 0); };
-  key_map[FKey::Page_up]   = [this] { scrollBy (0, -int(getViewportHeight())); };
-  key_map[FKey::Page_down] = [this] { scrollBy (0, int(getViewportHeight())); };
-  key_map[FKey::Home]      = [this] { scrollToY (1); };
-  key_map[FKey::End]       = \
-      [this] ()
-      {
-        auto yoffset_end = int(getScrollHeight() - getViewportHeight());
-        scrollToY (1 + yoffset_end);
-      };
+  auto scrollToEnd = [this] ()
+  {
+    auto yoffset_end = int(getScrollHeight() - getViewportHeight());
+    scrollToY (1 + yoffset_end);
+  };
+
+  key_map =
+  {
+    { FKey::Up        , [this] { scrollBy (0, -1); } },
+    { FKey::Down      , [this] { scrollBy (0, 1); } },
+    { FKey::Left      , [this] { scrollBy (-1, 0); } },
+    { FKey::Right     , [this] { scrollBy (1, 0); } },
+    { FKey::Page_up   , [this] { scrollBy (0, -int(getViewportHeight())); } },
+    { FKey::Page_down , [this] { scrollBy (0, int(getViewportHeight())); } },
+    { FKey::Home      , [this] { scrollToY (1); } },
+    { FKey::End       , scrollToEnd }
+  };
+}
+
+//----------------------------------------------------------------------
+void FScrollView::changeSize (const FSize& size, bool adjust)
+{
+  const std::size_t w = size.getWidth();
+  const std::size_t h = size.getHeight();
+  viewport_geometry.setSize ( w - vertical_border_spacing - nf_offset
+                            , h - horizontal_border_spacing );
+  calculateScrollbarPos();
+
+  if ( getScrollWidth() < getViewportWidth()
+    || getScrollHeight() < getViewportHeight() )
+  {
+    FScrollView::setScrollSize (getViewportSize());
+  }
+  else if ( ! adjust && viewport )
+  {
+    viewport->offset_left = scroll_geometry.getX();
+    viewport->offset_top = scroll_geometry.getY();
+  }
+
+  if ( ! viewport )
+    return;
+
+  // Insufficient space scrolling
+  viewport_geometry.x1_ref() = \
+      std::min ( int(getScrollWidth() - getViewportWidth())
+               , viewport_geometry.getX1() );
+  viewport_geometry.y1_ref() = \
+      std::min ( int(getScrollHeight() - getViewportHeight())
+               , viewport_geometry.getY1() );
 }
 
 //----------------------------------------------------------------------
@@ -855,8 +1008,8 @@ void FScrollView::setViewportCursor()
                           , viewport->input_cursor_y - 1 };
   const FPoint window_cursor_pos{ getViewportCursorPos() };
   auto printarea = getCurrentPrintArea();
-  printarea->input_cursor_x = window_cursor_pos.getX();
-  printarea->input_cursor_y = window_cursor_pos.getY();
+  printarea->setInputCursorPos ( window_cursor_pos.getX()
+                               , window_cursor_pos.getY() );
 
   if ( viewport->input_cursor_visible
     && viewport_geometry.contains(cursor_pos) )
@@ -868,7 +1021,7 @@ void FScrollView::setViewportCursor()
 //----------------------------------------------------------------------
 void FScrollView::cb_vbarChange (const FWidget*)
 {
-  FScrollbar::ScrollType scroll_type = vbar->getScrollType();
+  auto scroll_type = vbar->getScrollType();
   static constexpr int wheel_distance = 4;
   int distance{1};
 
@@ -910,7 +1063,7 @@ void FScrollView::cb_vbarChange (const FWidget*)
       break;
 
     default:
-      break;
+      throw std::invalid_argument{"Invalid scroll type"};
   }
 
   update_scrollbar = true;
@@ -919,7 +1072,7 @@ void FScrollView::cb_vbarChange (const FWidget*)
 //----------------------------------------------------------------------
 void FScrollView::cb_hbarChange (const FWidget*)
 {
-  FScrollbar::ScrollType scroll_type = hbar->getScrollType();
+  auto scroll_type = hbar->getScrollType();
   static constexpr int wheel_distance = 4;
   int distance{1};
 
@@ -961,10 +1114,17 @@ void FScrollView::cb_hbarChange (const FWidget*)
       break;
 
     default:
-      break;
+      throw std::invalid_argument{"Invalid scroll type"};
   }
 
   update_scrollbar = true;
 }
 
+// FVTerm friend function definition
+//----------------------------------------------------------------------
+void setPrintArea (FWidget& widget, FVTerm::FTermArea* area)
+{
+  widget.print_area = area;
+}
+
 }  // namespace finalcut
