@@ -3,7 +3,7 @@
 *                                                                      *
 * This file is part of the FINAL CUT widget toolkit                    *
 *                                                                      *
-* Copyright 2014-2021 Markus Gans                                      *
+* Copyright 2014-2023 Markus Gans                                      *
 *                                                                      *
 * FINAL CUT is free software; you can redistribute it and/or modify    *
 * it under the terms of the GNU Lesser General Public License as       *
@@ -20,6 +20,7 @@
 * <http://www.gnu.org/licenses/>.                                      *
 ***********************************************************************/
 
+#include <utility>
 #include <vector>
 
 #include "final/fevent.h"
@@ -88,7 +89,7 @@ void FStatusKey::setActive()
 }
 
 //----------------------------------------------------------------------
-bool FStatusKey::setMouseFocus(bool enable)
+auto FStatusKey::setMouseFocus(bool enable) -> bool
 {
   if ( mouse_focus == enable )
     return true;
@@ -101,7 +102,7 @@ bool FStatusKey::setMouseFocus(bool enable)
 //----------------------------------------------------------------------
 void FStatusKey::init()
 {
-  setGeometry (FPoint{1, 1}, FSize{1, 1});
+  FWidget::setGeometry (FPoint{1, 1}, FSize{1, 1});
   FWidget* parent = getParentWidget();
 
   if ( parent && parent->isInstanceOf("FStatusBar") )
@@ -169,7 +170,7 @@ void FStatusBar::resetColors()
 }
 
 //----------------------------------------------------------------------
-bool FStatusBar::hasActivatedKey() const
+auto FStatusBar::hasActivatedKey() const -> bool
 {
   if ( ! key_list.empty() )
   {
@@ -185,8 +186,8 @@ bool FStatusBar::hasActivatedKey() const
 void FStatusBar::hide()
 {
   const auto& wc = getColorTheme();
-  const FColor fg = wc->term_fg;
-  const FColor bg = wc->term_bg;
+  const auto& fg = wc->term_fg;
+  const auto& bg = wc->term_bg;
   setColor (fg, bg);
   print() << FPoint{1, 1} << FString{getDesktopWidth(), L' '};
   FWindow::hide();
@@ -389,101 +390,91 @@ void FStatusBar::onMouseDown (FMouseEvent* ev)
 //----------------------------------------------------------------------
 void FStatusBar::onMouseUp (FMouseEvent* ev)
 {
-  if ( hasActivatedKey() )
+  if ( hasActivatedKey()
+    || ev->getButton() != MouseButton::Left
+    || mouse_down == false
+    || key_list.empty() )
     return;
 
-  if ( ev->getButton() != MouseButton::Left )
-    return;
+  mouse_down = false;
+  int X{1};
+  auto iter = key_list.cbegin();
+  const auto& last = key_list.cend();
 
-  if ( mouse_down )
+  while ( iter != last )
   {
-    mouse_down = false;
+    const int x1 = X;
+    const int kname_len = getKeyNameWidth(*iter);
+    const int txt_length = getKeyTextWidth(*iter);
+    const int x2 = x1 + kname_len + txt_length + 1;
 
-    if ( ! key_list.empty() )
+    if ( (*iter)->hasMouseFocus() )
     {
-      int X{1};
-      auto iter = key_list.cbegin();
-      const auto& last = key_list.cend();
+      (*iter)->unsetMouseFocus();
+      const int mouse_x = ev->getX();
+      const int mouse_y = ev->getY();
 
-      while ( iter != last )
-      {
-        const int x1 = X;
-        const int kname_len = getKeyNameWidth(*iter);
-        const int txt_length = getKeyTextWidth(*iter);
-        const int x2 = x1 + kname_len + txt_length + 1;
+      if ( mouse_x >= x1 && mouse_x <= x2 && mouse_y == 1 )
+        (*iter)->setActive();
 
-        if ( (*iter)->hasMouseFocus() )
-        {
-          (*iter)->unsetMouseFocus();
-          const int mouse_x = ev->getX();
-          const int mouse_y = ev->getY();
-
-          if ( mouse_x >= x1 && mouse_x <= x2 && mouse_y == 1 )
-            (*iter)->setActive();
-
-          // unset after get back from callback
-          (*iter)->unsetActive();
-          redraw();
-        }
-
-        X = x2 + 2;
-        ++iter;
-      }
+      // unset after get back from callback
+      (*iter)->unsetActive();
+      redraw();
     }
+
+    X = x2 + 2;
+    ++iter;
   }
 }
 
 //----------------------------------------------------------------------
 void FStatusBar::onMouseMove (FMouseEvent* ev)
 {
-  if ( hasActivatedKey() )
+  if ( hasActivatedKey()
+    || ev->getButton() != MouseButton::Left
+    || mouse_down == false
+    || key_list.empty() )
     return;
 
-  if ( ev->getButton() != MouseButton::Left )
-    return;
+  bool focus_changed{false};
+  int X{1};
+  auto iter = key_list.cbegin();
+  const auto& last = key_list.cend();
 
-  if ( mouse_down && ! key_list.empty() )
+  while ( iter != last )
   {
-    bool focus_changed{false};
-    int X{1};
-    auto iter = key_list.cbegin();
-    const auto& last = key_list.cend();
+    const int x1 = X;
+    const int kname_len = getKeyNameWidth(*iter);
+    const int txt_length = getKeyTextWidth(*iter);
+    const int x2 = x1 + kname_len + txt_length + 1;
+    const int mouse_x = ev->getX();
+    const int mouse_y = ev->getY();
 
-    while ( iter != last )
+    if ( mouse_x >= x1
+      && mouse_x <= x2
+      && mouse_y == 1 )
     {
-      const int x1 = X;
-      const int kname_len = getKeyNameWidth(*iter);
-      const int txt_length = getKeyTextWidth(*iter);
-      const int x2 = x1 + kname_len + txt_length + 1;
-      const int mouse_x = ev->getX();
-      const int mouse_y = ev->getY();
-
-      if ( mouse_x >= x1
-        && mouse_x <= x2
-        && mouse_y == 1 )
+      if ( ! (*iter)->hasMouseFocus() )
       {
-        if ( ! (*iter)->hasMouseFocus() )
-        {
-          (*iter)->setMouseFocus();
-          focus_changed = true;
-        }
+        (*iter)->setMouseFocus();
+        focus_changed = true;
       }
-      else
+    }
+    else
+    {
+      if ( (*iter)->hasMouseFocus() )
       {
-        if ( (*iter)->hasMouseFocus() )
-        {
-          (*iter)->unsetMouseFocus();
-          focus_changed = true;
-        }
+        (*iter)->unsetMouseFocus();
+        focus_changed = true;
       }
-
-      X = x2 + 2;
-      ++iter;
     }
 
-    if ( focus_changed )
-      redraw();
+    X = x2 + 2;
+    ++iter;
   }
+
+  if ( focus_changed )
+    redraw();
 }
 
 //----------------------------------------------------------------------
@@ -517,7 +508,7 @@ void FStatusBar::init()
   const std::size_t w = r->getWidth();
   const auto h = int(r->getHeight());
   // initialize geometry values
-  setGeometry (FPoint{1, h}, FSize{w, 1}, false);
+  FWindow::setGeometry (FPoint{1, h}, FSize{w, 1}, false);
   setAlwaysOnTop();
   setStatusBar(this);
   ignorePadding();
@@ -531,14 +522,14 @@ void FStatusBar::init()
 }
 
 //----------------------------------------------------------------------
-int FStatusBar::getKeyNameWidth (const FStatusKey* key) const
+auto FStatusBar::getKeyNameWidth (const FStatusKey* key) const -> int
 {
   const FString& key_name = FVTerm::getFOutput()->getKeyName(key->getKey());
   return int(getColumnWidth(key_name));
 }
 
 //----------------------------------------------------------------------
-int FStatusBar::getKeyTextWidth (const FStatusKey* key) const
+auto FStatusBar::getKeyTextWidth (const FStatusKey* key) const -> int
 {
   const FString& key_text = key->getText();
   return int(getColumnWidth(key_text));
@@ -605,7 +596,7 @@ void FStatusBar::drawKey (FKeyList::const_iterator iter)
 {
   // Draw not active key
 
-  const auto item = *iter;
+  const auto& item = *iter;
   const auto& wc = getColorTheme();
   setColor (wc->statusbar_hotkey_fg, wc->statusbar_hotkey_bg);
   x++;
