@@ -3,7 +3,7 @@
 *                                                                      *
 * This file is part of the FINAL CUT widget toolkit                    *
 *                                                                      *
-* Copyright 2015-2021 Markus Gans                                      *
+* Copyright 2015-2023 Markus Gans                                      *
 *                                                                      *
 * FINAL CUT is free software; you can redistribute it and/or modify    *
 * it under the terms of the GNU Lesser General Public License as       *
@@ -24,6 +24,7 @@
 
 #include "final/fapplication.h"
 #include "final/fevent.h"
+#include "final/input/fmouse.h"
 #include "final/menu/fmenubar.h"
 #include "final/menu/fmenu.h"
 #include "final/widget/fcombobox.h"
@@ -33,10 +34,21 @@
 namespace finalcut
 {
 
+namespace internal
+{
+
+struct var
+{
+  static bool fwindow_init_flag;  // FWindow init state
+};
+
+bool var::fwindow_init_flag{false};
+
+}  // namespace internal
+
 // static attributes
 FWindow* FWindow::previous_window{nullptr};
 
-
 //----------------------------------------------------------------------
 // class FWindow
 //----------------------------------------------------------------------
@@ -46,10 +58,16 @@ FWindow* FWindow::previous_window{nullptr};
 FWindow::FWindow(FWidget* parent)
   : FWidget{parent}
 {
+  if ( ! internal::var::fwindow_init_flag )
+  {
+    auto app_object = FApplication::getApplicationObject();
+    app_object->registerMouseHandler (closeDropDownMouseHandler);
+    app_object->registerMouseHandler (unselectMenubarItemsMouseHandler);
+    internal::var::fwindow_init_flag = true;
+  }
+
   setWindowWidget();
-  FRect geometry {getTermGeometry()};
-  geometry.move(-1, -1);
-  createArea (geometry, getShadow(), getVWin());
+  createVWin();
   addWindow (this);
 }
 
@@ -73,26 +91,24 @@ FWindow::~FWindow()  // destructor
     const auto& t_geometry = getTermGeometryWithShadow();
     restoreVTerm (t_geometry);
   }
-
-  removeArea (getVWin());
 }
 
 
 // public methods of FWindow
 //----------------------------------------------------------------------
-FWidget* FWindow::getWindowFocusWidget() const
+auto FWindow::getWindowFocusWidget() const -> FWidget*
 {
   // returns the focused widget of this window
   return win_focus_widget;
 }
 
 //----------------------------------------------------------------------
-bool FWindow::setWindowWidget (bool enable)
+auto FWindow::setWindowWidget (bool enable) -> bool
 {
   if ( isWindowWidget() == enable )
     return true;
 
-  setFlags().window_widget = enable;
+  setFlags().type.window_widget = enable;
 
   if ( enable )
     setTermOffset();
@@ -143,7 +159,7 @@ void FWindow::setWindowFocusWidget (FWidget* obj)
 }
 
 //----------------------------------------------------------------------
-bool FWindow::activateWindow (bool enable)
+auto FWindow::activateWindow (bool enable) -> bool
 {
   // activate/deactivate this window
   if ( enable )
@@ -163,21 +179,21 @@ void FWindow::unsetActiveWindow() const
 }
 
 //----------------------------------------------------------------------
-bool FWindow::setResizeable (bool enable)
+auto FWindow::setResizeable (bool enable) -> bool
 {
-  return (setFlags().resizeable = enable);
+  return (setFlags().feature.resizeable = enable);
 }
 
 //----------------------------------------------------------------------
-bool FWindow::setMinimizable (bool enable)
+auto FWindow::setMinimizable (bool enable) -> bool
 {
-  return (setFlags().minimizable = enable);
+  return (setFlags().feature.minimizable = enable);
 }
 
 //----------------------------------------------------------------------
-bool FWindow::setTransparentShadow (bool enable)
+auto FWindow::setTransparentShadow (bool enable) -> bool
 {
-  setFlags().shadow = setFlags().trans_shadow = enable;
+  setFlags().shadow.shadow = setFlags().shadow.trans_shadow = enable;
 
   if ( enable )
     setShadowSize (FSize{2, 1});
@@ -188,21 +204,21 @@ bool FWindow::setTransparentShadow (bool enable)
 }
 
 //----------------------------------------------------------------------
-bool FWindow::setShadow (bool enable)
+auto FWindow::setShadow (bool enable) -> bool
 {
   if ( FVTerm::getFOutput()->isMonochron() )
     return false;
 
   if ( enable )
   {
-    setFlags().shadow = true;
-    setFlags().trans_shadow = false;
+    setFlags().shadow.shadow = true;
+    setFlags().shadow.trans_shadow = false;
     setShadowSize (FSize{1, 1});
   }
   else
   {
-    setFlags().shadow = false;
-    setFlags().trans_shadow = false;
+    setFlags().shadow.shadow = false;
+    setFlags().shadow.trans_shadow = false;
     setShadowSize (FSize{0, 0});
   }
 
@@ -210,12 +226,12 @@ bool FWindow::setShadow (bool enable)
 }
 
 //----------------------------------------------------------------------
-bool FWindow::setAlwaysOnTop (bool enable)
+auto FWindow::setAlwaysOnTop (bool enable) -> bool
 {
   if ( isAlwaysOnTop() == enable )
     return true;
 
-  setFlags().always_on_top = enable;
+  setFlags().visibility.always_on_top = enable;
 
   if ( enable )
   {
@@ -234,7 +250,7 @@ bool FWindow::setAlwaysOnTop (bool enable)
 }
 
 //----------------------------------------------------------------------
-bool FWindow::isMinimized() const
+auto FWindow::isMinimized() const -> bool
 {
   // returns the window minimized state
 
@@ -245,7 +261,7 @@ bool FWindow::isMinimized() const
 }
 
 //----------------------------------------------------------------------
-bool FWindow::isWindowHidden() const
+auto FWindow::isWindowHidden() const -> bool
 {
   // returns the window hidden state
 
@@ -443,7 +459,7 @@ void FWindow::move (const FPoint& pos)
 }
 
 //----------------------------------------------------------------------
-FWindow* FWindow::getWindowWidgetAt (int x, int y)
+auto FWindow::getWindowWidgetAt (int x, int y) -> FWindow*
 {
   // returns the window object to the corresponding coordinates
 
@@ -508,8 +524,8 @@ void FWindow::swapWindow (const FWidget* obj1, const FWidget* obj2)
 
   if ( ! getWindowList()
     || getWindowList()->empty()
-    || obj1->getFlags().modal
-    || obj2->getFlags().modal )
+    || obj1->getFlags().visibility.modal
+    || obj2->getFlags().visibility.modal )
     return;
 
   auto iter = getWindowList()->cbegin();
@@ -535,7 +551,7 @@ void FWindow::swapWindow (const FWidget* obj1, const FWidget* obj2)
 }
 
 //----------------------------------------------------------------------
-bool FWindow::raiseWindow (FWidget* obj)
+auto FWindow::raiseWindow (FWidget* obj) -> bool
 {
   // raises the window widget obj to the top
 
@@ -546,7 +562,7 @@ bool FWindow::raiseWindow (FWidget* obj)
 
   const auto last = static_cast<FWidget*>(getWindowList()->back());
 
-  if ( last == obj || (last->getFlags().modal && ! obj->isMenuWidget()) )
+  if ( last == obj || (last->getFlags().visibility.modal && ! obj->isMenuWidget()) )
     return false;
 
   auto iter = getWindowList()->cbegin();
@@ -570,7 +586,7 @@ bool FWindow::raiseWindow (FWidget* obj)
 }
 
 //----------------------------------------------------------------------
-bool FWindow::lowerWindow (FWidget* obj)
+auto FWindow::lowerWindow (FWidget* obj) -> bool
 {
   // lowers the window widget obj to the bottom
 
@@ -578,7 +594,7 @@ bool FWindow::lowerWindow (FWidget* obj)
     || getWindowList()->empty()
     || ! obj->isWindowWidget()
     || getWindowList()->front() == obj
-    || obj->getFlags().modal )
+    || obj->getFlags().visibility.modal )
     return false;
 
   auto iter = getWindowList()->cbegin();
@@ -602,15 +618,15 @@ bool FWindow::lowerWindow (FWidget* obj)
 }
 
 //----------------------------------------------------------------------
-bool FWindow::zoomWindow()
+auto FWindow::zoomWindow() -> bool
 {
   if ( zoomed )
   {
     zoomed = false;
     const FRect oldGeometry (getTermGeometryWithShadow());
     setGeometry (normalGeometry);
-    restoreVTerm (oldGeometry);
     redraw();
+    restoreVTerm (oldGeometry);
   }
   else
   {
@@ -622,15 +638,15 @@ bool FWindow::zoomWindow()
     normalGeometry = getGeometry();
     const FRect oldGeometry (getTermGeometryWithShadow());
     setGeometry (FPoint{1, 1}, FSize{getMaxWidth(), getMaxHeight()});
-    restoreVTerm (oldGeometry);
     redraw();
+    restoreVTerm (oldGeometry);
   }
 
   return zoomed;
 }
 
 //----------------------------------------------------------------------
-bool FWindow::minimizeWindow()
+auto FWindow::minimizeWindow() -> bool
 {
   if ( ! isVirtualWindow() )
     return false;
@@ -676,28 +692,14 @@ void FWindow::switchToPrevWindow (const FWidget* widget)
         && w != static_cast<FWindow*>(getStatusBar())
         && w != static_cast<FWindow*>(getMenuBar()) )
       {
-        setActiveWindow(w);
+        FWindow::setActiveWindow(w);
         break;
       }
     }
     while ( iter != begin );
   }
 
-  if ( active_win )
-  {
-    auto focus = active_win->getWindowFocusWidget();
-
-    if ( ! active_win->isWindowActive() )
-      FWidget::setActiveWindow(active_win);
-
-    if ( focus )
-    {
-      focus->setFocus();
-
-      if ( ! focus->isWindowWidget() )
-        focus->redraw();
-    }
-  }
+  reactivateWindow (active_win);
 
   // Enable terminal updates again
   if ( widget )
@@ -705,7 +707,7 @@ void FWindow::switchToPrevWindow (const FWidget* widget)
 }
 
 //----------------------------------------------------------------------
-bool FWindow::activatePrevWindow()
+auto FWindow::activatePrevWindow() -> bool
 {
   // activate the previous window
   const auto& w = previous_window;
@@ -717,7 +719,7 @@ bool FWindow::activatePrevWindow()
 
     if ( ! w->isWindowHidden() )
     {
-      FWidget::setActiveWindow(w);
+      FWindow::setActiveWindow(w);
       return true;
     }
   }
@@ -762,7 +764,7 @@ void FWindow::adjustSize()
 }
 
 //----------------------------------------------------------------------
-bool FWindow::event (FEvent* ev)
+auto FWindow::event (FEvent* ev) -> bool
 {
   auto event_type = ev->getType();
 
@@ -819,7 +821,17 @@ void FWindow::onWindowLowered (FEvent*)
 
 // private methods of FWindow
 //----------------------------------------------------------------------
-inline FRect FWindow::getVisibleTermGeometry (FWindow* win)
+inline void FWindow::createVWin() noexcept
+{
+  // Initialize virtual window
+
+  FRect geometry {getTermGeometry()};
+  geometry.move(-1, -1);
+  setVWin(createArea(geometry, getShadow()));
+}
+
+//----------------------------------------------------------------------
+inline auto FWindow::getVisibleTermGeometry (FWindow* win) -> FRect
 {
   auto& term_geometry = win->getTermGeometry();
 
@@ -881,7 +893,7 @@ void FWindow::processAlwaysOnTop()
 }
 
 //----------------------------------------------------------------------
-FWindow* FWindow::getWindowWidgetImpl (FWidget* obj)
+auto FWindow::getWindowWidgetImpl (FWidget* obj) -> FWindow*
 {
   // returns the window object to the given widget obj
   auto p_obj = obj->getParentWidget();
@@ -899,7 +911,7 @@ FWindow* FWindow::getWindowWidgetImpl (FWidget* obj)
 }
 
 //----------------------------------------------------------------------
-int FWindow::getWindowLayerImpl (FWidget* obj)
+auto FWindow::getWindowLayerImpl (FWidget* obj) -> int
 {
   // returns the window layer from the widget obj
 
@@ -916,7 +928,46 @@ int FWindow::getWindowLayerImpl (FWidget* obj)
   return FVTerm::getLayer(*window);
 }
 
+//----------------------------------------------------------------------
+void FWindow::reactivateWindow (FWindow* active_win)
+{
+  if ( ! active_win )
+    return;
+
+  auto focus = active_win->getWindowFocusWidget();
+
+  if ( ! active_win->isWindowActive() )
+    FWindow::setActiveWindow(active_win);
+
+  if ( focus && ! focus->isInstanceOf("FMenuItem") )
+  {
+    // Renew the focus of the focused widget in the current window
+    auto last_focus = FWidget::getFocusWidget();
+
+    if ( last_focus )
+      last_focus->setFlags().focus.focus = false;
+
+    FWidget::setFocusWidget(focus);
+    focus->setFlags().focus.focus = true;
+    active_win->setWindowFocusWidget (focus);
+    FFocusEvent f_in (Event::FocusIn);
+    FApplication::sendEvent (focus, &f_in);
+  }
+}
+
+
 // non-member functions
+//----------------------------------------------------------------------
+void closeDropDownMouseHandler (const FMouseData& md)
+{
+  if ( md.isMoved() )
+    return;
+
+  const auto& mouse_position = md.getPos();
+  auto app_object = FApplication::getApplicationObject();
+  finalcut::closeDropDown (app_object, mouse_position);
+}
+
 //----------------------------------------------------------------------
 void closeDropDown (const FWidget* widget, const FPoint& mouse_position)
 {
@@ -952,8 +1003,43 @@ void closeDropDown (const FWidget* widget, const FPoint& mouse_position)
   if ( ! (FWidget::getClickedWidget() || is_dialog_menu) )
     FWindow::switchToPrevWindow(widget);
 
+  drawStatusBarMessage();
+}
+
+//----------------------------------------------------------------------
+void unselectMenubarItemsMouseHandler (const FMouseData& md)
+{
+  if ( md.isMoved() )
+    return;
+
+  const auto& mouse_position = md.getPos();
+  auto app_object = FApplication::getApplicationObject();
+  finalcut::unselectMenubarItems (app_object, mouse_position);
+}
+
+//----------------------------------------------------------------------
+void unselectMenubarItems (const FWidget* widget, const FPoint& mouse_position)
+{
+  // Unselect the menu bar items
+
+  auto menu_bar = FWidget::getMenuBar();
+
+  if ( FWidget::getOpenMenu()
+    || ! (menu_bar && menu_bar->hasSelectedItem())
+    || menu_bar->getTermGeometry().contains(mouse_position) )
+    return;
+
   if ( FWidget::getStatusBar() )
-    FWidget::getStatusBar()->drawMessage();
+    FWidget::getStatusBar()->clearMessage();
+
+  menu_bar->resetMenu();
+  menu_bar->redraw();
+
+  // No widget was been clicked
+  if ( ! FWidget::getClickedWidget() )
+    FWindow::switchToPrevWindow(widget);
+
+  drawStatusBarMessage();
 }
 
 }  // namespace finalcut
