@@ -8,9 +8,15 @@ else
   shift
 fi
 
-# Is the file executable?
-test ! -x "$PROG" && echo "No executable file not found" && exit 1
+error_handling ()
+{
+  echo "No ELF executable!"
+  exit 1
+}
 
+# Is the file executable?
+test -x "$PROG" || error_handling
+file --brief "$PROG" | grep -q "ELF" || error_handling
 
 LD_LIBRARY_PATH=../final/.libs/ valgrind --tool=callgrind -v "$PROG" "$@" 2>/dev/null
 kcachegrind
