@@ -3,7 +3,7 @@
 *                                                                      *
 * This file is part of the FINAL CUT widget toolkit                    *
 *                                                                      *
-* Copyright 2019-2021 Markus Gans                                      *
+* Copyright 2019-2022 Markus Gans                                      *
 *                                                                      *
 * FINAL CUT is free software; you can redistribute it and/or modify    *
 * it under the terms of the GNU Lesser General Public License as       *
@@ -34,6 +34,9 @@
 #include <sys/wait.h>
 #include <sys/mman.h>
 
+#include <chrono>
+#include <thread>
+
 #include <final/final.h>
 
 using finalcut::C_STR;
@@ -63,6 +66,7 @@ class ConEmu
       tera_term,
       cygwin,
       mintty,
+      stterm,
       linux_con,
       freebsd_con,
       netbsd_con,
@@ -111,36 +115,36 @@ class ConEmu
     }
 
     // Disable assignment operator (=)
-    ConEmu& operator = (const ConEmu&) = delete;
+    auto operator = (const ConEmu&) -> ConEmu& = delete;
 
   protected:
     // Mutators
-    void        enableConEmuDebug (bool) noexcept;
+    void  enableConEmuDebug (bool) noexcept;
 
     // Inquiries
-    bool        isConEmuChildProcess (pid_t) const noexcept;
+    auto  isConEmuChildProcess (pid_t) const noexcept -> bool;
 
     // Methods
-    void        printConEmuDebug();
-    void        closeConEmuStdStreams();
-    pid_t       forkConEmu();
-    void        startConEmuTerminal (console);
+    void  printConEmuDebug();
+    void  closeConEmuStdStreams();
+    auto  forkConEmu() -> pid_t;
+    void  startConEmuTerminal (console);
 
   private:
     // Accessors
-    const char* getAnswerback (console);
-    const char* getDSR (console);
-    const char* getDECID (console);
-    const char* getDA (console);
-    const char* getDA1 (console);
-    const char* getSEC_DA (console);
+    auto  getAnswerback (console) -> const char*;
+    auto  getDSR (console) -> const char*;
+    auto  getDECID (console) -> const char*;
+    auto  getDA (console) -> const char*;
+    auto  getDA1 (console) -> const char*;
+    auto  getSEC_DA (console) -> const char*;
 
     // Methods
-    bool        openMasterPTY();
-    bool        openSlavePTY();
-    void        closeMasterPTY();
-    void        closeSlavePTY();
-    void        parseTerminalBuffer (std::size_t, console);
+    auto  openMasterPTY() -> bool;
+    auto  openSlavePTY() -> bool;
+    void  closeMasterPTY();
+    void  closeSlavePTY();
+    void  parseTerminalBuffer (std::size_t, console);
 
     // Data members
     int                fd_stdin{fileno(stdin)};
@@ -432,7 +436,7 @@ inline void ConEmu::enableConEmuDebug (bool enable) noexcept
 }
 
 //----------------------------------------------------------------------
-inline bool ConEmu::isConEmuChildProcess (pid_t pid) const noexcept
+inline auto ConEmu::isConEmuChildProcess (pid_t pid) const noexcept -> bool
 {
   return bool( pid == 0 );
 }
@@ -488,10 +492,10 @@ inline void ConEmu::closeConEmuStdStreams()
 }
 
 //----------------------------------------------------------------------
-inline pid_t ConEmu::forkConEmu()
+inline auto ConEmu::forkConEmu() -> pid_t
 {
   // Initialize buffer with '\0'
-  std::fill_n (buffer, sizeof(buffer), '\0');
+  std::fill (buffer, buffer + sizeof(buffer), '\0');
 
   if ( ! openMasterPTY() )
     return -1;
@@ -564,9 +568,8 @@ inline pid_t ConEmu::forkConEmu()
     // Wait until the child process is ready for input
     while ( ! *shared_state && i < timeout )
     {
-      // Wait 10 ms (= 10,000,000 ns)
-      const struct timespec ms[]{{0, 10000000L}};
-      nanosleep (ms, nullptr);
+      // Wait 10 ms
+      std::this_thread::sleep_for(std::chrono::milliseconds(10));
       i++;
     }
 
@@ -576,8 +579,6 @@ inline pid_t ConEmu::forkConEmu()
   return pid;
 }
 
-
-
 //----------------------------------------------------------------------
 inline void ConEmu::startConEmuTerminal (console con)
 {
@@ -585,8 +586,10 @@ inline void ConEmu::startConEmuTerminal (console con)
     return;
 
   closeSlavePTY();
+  uInt64 timeout = 10'000'000;  // 10 seconds
+  TimeValue time_last_data = finalcut::FObjectTimer::getCurrentTime();
 
-  while ( true )
+  while ( ! finalcut::FObjectTimer::isTimeout (time_last_data, timeout) )
   {
     fd_set ifds;
     struct timeval tv;
@@ -611,6 +614,7 @@ inline void ConEmu::startConEmuTerminal (console con)
       {
         buffer[len] = '\0';
         write (fd_master, buffer, len);  // Send data to the master side
+        time_last_data = finalcut::FObjectTimer::getCurrentTime();
       }
     }
 
@@ -626,6 +630,7 @@ inline void ConEmu::startConEmuTerminal (console con)
       {
         buffer[len] = '\0';
         parseTerminalBuffer (len, con);
+        time_last_data = finalcut::FObjectTimer::getCurrentTime();
       }
     }
   }
@@ -634,7 +639,7 @@ inline void ConEmu::startConEmuTerminal (console con)
 
 // private methods of ConEmu
 //----------------------------------------------------------------------
-inline const char* ConEmu::getAnswerback (console con)
+inline auto ConEmu::getAnswerback (console con) -> const char*
 {
   static const char* Answerback[] =
   {
@@ -650,6 +655,7 @@ inline const char* ConEmu::getAnswerback (console con)
     nullptr,         // Tera Term
     nullptr,         // Cygwin
     nullptr,         // Mintty
+    nullptr,         // st - simple terminal
     nullptr,         // Linux console
     nullptr,         // FreeBSD console
     nullptr,         // NetBSD console
@@ -666,7 +672,7 @@ inline const char* ConEmu::getAnswerback (console con)
 }
 
 //----------------------------------------------------------------------
-inline const char* ConEmu::getDSR (console con)
+inline auto ConEmu::getDSR (console con) -> const char*
 {
   static const char* DSR[] =
   {
@@ -682,6 +688,7 @@ inline const char* ConEmu::getDSR (console con)
     C_STR("\033[0n"),  // Tera Term
     nullptr,           // Cygwin
     C_STR("\033[0n"),  // Mintty
+    nullptr,           // st - simple terminal
     C_STR("\033[0n"),  // Linux console
     C_STR("\033[0n"),  // FreeBSD console
     C_STR("\033[0n"),  // NetBSD console
@@ -698,7 +705,7 @@ inline const char* ConEmu::getDSR (console con)
 }
 
 //----------------------------------------------------------------------
-inline const char* ConEmu::getDECID (console con)
+inline auto ConEmu::getDECID (console con) -> const char*
 {
   static const char* DECID[] =
   {
@@ -714,6 +721,7 @@ inline const char* ConEmu::getDECID (console con)
     C_STR("\033[?1;2c"),                   // Tera Term
     nullptr,                               // Cygwin
     C_STR("\033[?1;2;6;22c"),              // Mintty
+    C_STR("\033[?6c"),                     // st - simple terminal
     C_STR("\033[?6c"),                     // Linux console
     nullptr,                               // FreeBSD console
     nullptr,                               // NetBSD console
@@ -730,7 +738,7 @@ inline const char* ConEmu::getDECID (console con)
 }
 
 //----------------------------------------------------------------------
-inline const char* ConEmu::getDA (console con)
+inline auto ConEmu::getDA (console con) -> const char*
 {
   static const char* DA[] =
   {
@@ -746,6 +754,7 @@ inline const char* ConEmu::getDA (console con)
     C_STR("\033[?1;2c"),                   // Tera Term
     C_STR("\033[?6c"),                     // Cygwin
     C_STR("\033[?1;2;6;22c"),              // Mintty
+    C_STR("\033[?6c"),                     // st - simple terminal
     C_STR("\033[?6c"),                     // Linux console
     C_STR("\033[?1;2c"),                   // FreeBSD console
     C_STR("\033[?62;6c"),                  // NetBSD console
@@ -762,7 +771,7 @@ inline const char* ConEmu::getDA (console con)
 }
 
 //----------------------------------------------------------------------
-inline const char* ConEmu::getDA1 (console con)
+inline auto ConEmu::getDA1 (console con) -> const char*
 {
   static const char* DA1[] =
   {
@@ -778,6 +787,7 @@ inline const char* ConEmu::getDA1 (console con)
     C_STR("\033[?1;2c"),              // Tera Term
     C_STR("\033[?6c"),                // Cygwin
     C_STR("\033[?1;2;6;22c"),         // Mintty
+    nullptr,                          // st - simple terminal
     nullptr,                          // Linux console
     nullptr,                          // FreeBSD console
     nullptr,                          // NetBSD console
@@ -794,7 +804,7 @@ inline const char* ConEmu::getDA1 (console con)
 }
 
 //----------------------------------------------------------------------
-inline const char* ConEmu::getSEC_DA (console con)
+inline auto ConEmu::getSEC_DA (console con) -> const char*
 {
   static const char* SEC_DA[] =
   {
@@ -810,6 +820,7 @@ inline const char* ConEmu::getSEC_DA (console con)
     C_STR("\033[>32;278;0c"),     // Tera Term
     C_STR("\033[>67;200502;0c"),  // Cygwin
     C_STR("\033[>77;20402;0c"),   // Mintty
+    nullptr,                      // st - simple terminal
     nullptr,                      // Linux console
     C_STR("\033[>0;10;0c"),       // FreeBSD console
     C_STR("\033[>24;20;0c"),      // NetBSD console
@@ -826,7 +837,7 @@ inline const char* ConEmu::getSEC_DA (console con)
 }
 
 //----------------------------------------------------------------------
-inline bool ConEmu::openMasterPTY()
+inline auto ConEmu::openMasterPTY() -> bool
 {
   int result;
 
@@ -852,7 +863,7 @@ inline bool ConEmu::openMasterPTY()
 }
 
 //----------------------------------------------------------------------
-inline bool ConEmu::openSlavePTY()
+inline auto ConEmu::openSlavePTY() -> bool
 {
   closeSlavePTY();
 
@@ -992,6 +1003,7 @@ inline void ConEmu::parseTerminalBuffer (std::size_t length, console con)
              && con != console::cygwin
              && con != console::win_terminal
              && con != console::mintty
+             && con != console::stterm
              && con != console::linux_con
              && con != console::freebsd_con
              && con != console::netbsd_con
@@ -1021,6 +1033,7 @@ inline void ConEmu::parseTerminalBuffer (std::size_t length, console con)
         && con != console::cygwin
         && con != console::win_terminal
         && con != console::mintty
+        && con != console::stterm
         && con != console::linux_con
         && con != console::freebsd_con
         && con != console::netbsd_con
@@ -1057,6 +1070,7 @@ inline void ConEmu::parseTerminalBuffer (std::size_t length, console con)
         && con != console::cygwin
         && con != console::win_terminal
         && con != console::mintty
+        && con != console::stterm
         && con != console::linux_con
         && con != console::freebsd_con
         && con != console::netbsd_con
@@ -1096,6 +1110,7 @@ inline void ConEmu::parseTerminalBuffer (std::size_t length, console con)
         && con != console::cygwin
         && con != console::win_terminal
         && con != console::mintty
+        && con != console::stterm
         && con != console::linux_con
         && con != console::freebsd_con
         && con != console::netbsd_con
