@@ -3,7 +3,7 @@
 *                                                                      *
 * This file is part of the FINAL CUT widget toolkit                    *
 *                                                                      *
-* Copyright 2019-2021 Markus Gans                                      *
+* Copyright 2019-2022 Markus Gans                                      *
 *                                                                      *
 * FINAL CUT is free software; you can redistribute it and/or modify    *
 * it under the terms of the GNU Lesser General Public License as       *
@@ -52,24 +52,24 @@ class FSystemTest : public finalcut::FSystem
     FSystemTest() = default;
 
     // Methods
-    uChar            inPortByte (uShort) override;
+    auto            inPortByte (uShort) -> uChar override;
     void             outPortByte (uChar, uShort) override;
-    int              isTTY (int) const override;
-    int              ioctl (int, uLong, ...) override;
-    int              open (const char*, int, ...) override;
-    int              close (int) override;
-    FILE*            fopen (const char*, const char*) override;
-    int              fclose (FILE*) override;
-    int              fputs (const char*, FILE*) override;
-    int              putchar (int) override;
-    uid_t            getuid() override;
-    uid_t            geteuid() override;
-    int              getpwuid_r (uid_t, struct passwd*, char*
-                                , size_t, struct passwd** ) override;
-    char*            realpath (const char*, char*) override;
-    std::string&     getCharacters();
-    int&             getCursorType();
-    struct keymap_t& getTerminalKeymap();
+    auto              isTTY (int) const -> int override;
+    auto              ioctl (int, uLong, ...) -> int override;
+    auto              open (const char*, int, ...) -> int override;
+    auto              close (int) -> int override;
+    auto            fopen (const char*, const char*) -> FILE* override;
+    auto              fclose (FILE*) -> int override;
+    auto              fputs (const char*, FILE*) -> int override;
+    auto              putchar (int) -> int override;
+    auto            getuid() -> uid_t override;
+    auto            geteuid() -> uid_t override;
+    auto              getpwuid_r (uid_t, struct passwd*, char*
+                                , size_t, struct passwd** ) -> int override;
+    auto            realpath (const char*, char*) -> char* override;
+    auto     getCharacters() -> std::string&;
+    auto             getCursorType() -> int&;
+    auto getTerminalKeymap() -> struct keymap_t&;
 
   private:
     // Data members
@@ -370,7 +370,7 @@ keymap_t FSystemTest::terminal_keymap{};
 
 // public methods of FSystemTest
 //----------------------------------------------------------------------
-uChar FSystemTest::inPortByte (uShort)
+auto FSystemTest::inPortByte (uShort) -> uChar
 {
   return 0;
 }
@@ -381,14 +381,14 @@ void FSystemTest::outPortByte (uChar, uShort)
 }
 
 //----------------------------------------------------------------------
-int FSystemTest::isTTY (int fd) const
+auto FSystemTest::isTTY (int fd) const -> int
 {
   std::cerr << "Call: isatty (fd=" << fd << ")\n";
   return 1;
 }
 
 //----------------------------------------------------------------------
-int FSystemTest::ioctl (int fd, uLong request, ...)
+auto FSystemTest::ioctl (int fd, uLong request, ...) -> int
 {
   va_list args{};
   void* argp{};
@@ -468,7 +468,7 @@ int FSystemTest::ioctl (int fd, uLong request, ...)
 }
 
 //----------------------------------------------------------------------
-int FSystemTest::open (const char* pathname, int flags, ...)
+auto FSystemTest::open (const char* pathname, int flags, ...) -> int
 {
   va_list args{};
   va_start (args, flags);
@@ -483,14 +483,14 @@ int FSystemTest::open (const char* pathname, int flags, ...)
 }
 
 //----------------------------------------------------------------------
-int FSystemTest::close (int fildes)
+auto FSystemTest::close (int fildes) -> int
 {
   std::cerr << "Call: close (fildes=" << fildes << ")\n";
   return 0;
 }
 
 //----------------------------------------------------------------------
-FILE* FSystemTest::fopen (const char* path, const char* mode)
+auto FSystemTest::fopen (const char* path, const char* mode) -> FILE*
 {
   std::cerr << "Call: fopen (path=" << path
             << ", mode=" << mode << ")\n";
@@ -498,14 +498,14 @@ FILE* FSystemTest::fopen (const char* path, const char* mode)
 }
 
 //----------------------------------------------------------------------
-int FSystemTest::fclose (FILE* fp)
+auto FSystemTest::fclose (FILE* fp) -> int
 {
   std::cerr << "Call: fclose (fp=" << fp << ")\n";
   return 0;
 }
 
 //----------------------------------------------------------------------
-int FSystemTest::fputs (const char* str, FILE* stream)
+auto FSystemTest::fputs (const char* str, FILE* stream) -> int
 {
   std::cerr << "Call: fputs (" << str << ", " << stream << ")\n";
   std::string string = str;
@@ -521,7 +521,7 @@ int FSystemTest::fputs (const char* str, FILE* stream)
 }
 
 //----------------------------------------------------------------------
-int FSystemTest::putchar (int c)
+auto FSystemTest::putchar (int c) -> int
 {
   std::cerr << "Call: putchar (" << c << ")\n";
   characters.push_back(c);
@@ -529,44 +529,44 @@ int FSystemTest::putchar (int c)
 }
 
 //----------------------------------------------------------------------
-uid_t FSystemTest::getuid()
+auto FSystemTest::getuid() -> uid_t
 {
   return 0;
 }
 
 //----------------------------------------------------------------------
-uid_t FSystemTest::geteuid()
+auto FSystemTest::geteuid() -> uid_t
 {
   return 0;
 }
 
 //----------------------------------------------------------------------
-int FSystemTest::getpwuid_r ( uid_t, struct passwd*, char*
-                            , size_t, struct passwd** )
+auto FSystemTest::getpwuid_r ( uid_t, struct passwd*, char*
+                            , size_t, struct passwd** ) -> int
 {
   return 0;
 }
 
 //----------------------------------------------------------------------
-char* FSystemTest::realpath (const char*, char*)
+auto FSystemTest::realpath (const char*, char*) -> char*
 {
   return const_cast<char*>("");
 }
 
 //----------------------------------------------------------------------
-std::string& FSystemTest::getCharacters()
+auto FSystemTest::getCharacters() -> std::string&
 {
   return characters;
 }
 
 //----------------------------------------------------------------------
-int& FSystemTest::getCursorType()
+auto FSystemTest::getCursorType() -> int&
 {
   return cursor_type;
 }
 
 //----------------------------------------------------------------------
-struct keymap_t& FSystemTest::getTerminalKeymap()
+auto FSystemTest::getTerminalKeymap() -> struct keymap_t&
 {
   return terminal_keymap;
 }
@@ -578,14 +578,14 @@ struct keymap_t& FSystemTest::getTerminalKeymap()
 // class ftermfreebsdTest
 //----------------------------------------------------------------------
 
-class ftermfreebsdTest : public CPPUNIT_NS::TestFixture, test::ConEmu
+class ftermfreebsdTest : public CPPUNIT_NS::TestFixture
+                       , test::ConEmu
 {
   public:
     ftermfreebsdTest() = default;
 
   protected:
     void classNameTest();
-    void netbsdConsoleTest();
     void freebsdConsoleTest();
 
   private:
@@ -598,8 +598,8 @@ class ftermfreebsdTest : public CPPUNIT_NS::TestFixture, test::ConEmu
 
     // End of test suite definition
     CPPUNIT_TEST_SUITE_END();
-    wchar_t charEncode (finalcut::UniChar);
-    wchar_t charEncode (wchar_t);
+    auto charEncode (finalcut::UniChar) -> wchar_t;
+    auto charEncode (wchar_t) -> wchar_t;
 };
 
 //----------------------------------------------------------------------
@@ -617,7 +617,7 @@ void ftermfreebsdTest::freebsdConsoleTest()
   setenv ("COLUMNS", "80", 1);
   setenv ("LINES", "25", 1);
 
-  std::unique_ptr<finalcut::FSystem> fsys = finalcut::make_unique<test::FSystemTest>();
+  std::unique_ptr<finalcut::FSystem> fsys = std::make_unique<test::FSystemTest>();
   finalcut::FTerm::setFSystem(fsys);
   std::cout << "\n";
   auto& data = finalcut::FTermData::getInstance();
@@ -647,10 +647,14 @@ void ftermfreebsdTest::freebsdConsoleTest()
   data.setVGAFont (false);
   data.setMonochron (false);
   data.setTermResized (false);
+
   // setupterm is needed for tputs in ncurses >= 6.1
   setupterm (static_cast<char*>(nullptr), 1, static_cast<int*>(nullptr));
+
   auto& term_detection = finalcut::FTermDetection::getInstance();
   term_detection.setTerminalDetection(true);
+  finalcut::FTermcap::init();
+
   pid_t pid = forkConEmu();
 
   if ( isConEmuChildProcess(pid) )
@@ -784,6 +788,7 @@ void ftermfreebsdTest::freebsdConsoleTest()
 
     std::string& characters = fsystest->getCharacters();
     characters.clear();
+
     freebsd.setBeep (20, 100);     // Hz < 21
     CPPUNIT_ASSERT ( characters.empty() );
     freebsd.setBeep (32767, 100);  // Hz > 32766
@@ -796,6 +801,7 @@ void ftermfreebsdTest::freebsdConsoleTest()
 
     CPPUNIT_ASSERT ( characters == CSI "=5965;10B" );
     characters.clear();
+
     freebsd.resetBeep();
     CPPUNIT_ASSERT ( characters == CSI "=800;5B" );
     characters.clear();
@@ -809,20 +815,24 @@ void ftermfreebsdTest::freebsdConsoleTest()
   {
     // Start the terminal emulation
     startConEmuTerminal (ConEmu::console::freebsd_con);
+    int wstatus;
 
-    if ( waitpid(pid, nullptr, WUNTRACED) != pid )
+    if ( waitpid(pid, &wstatus, WUNTRACED) != pid )
       std::cerr << "waitpid error" << std::endl;
+
+    if ( WIFEXITED(wstatus) )
+      CPPUNIT_ASSERT ( WEXITSTATUS(wstatus) == 0 );
   }
 }
 
 //----------------------------------------------------------------------
-wchar_t ftermfreebsdTest::charEncode (finalcut::UniChar c)
+auto ftermfreebsdTest::charEncode (finalcut::UniChar c) -> wchar_t
 {
   return charEncode(static_cast<wchar_t>(c));
 }
 
 //----------------------------------------------------------------------
-wchar_t ftermfreebsdTest::charEncode (wchar_t c)
+auto ftermfreebsdTest::charEncode (wchar_t c) -> wchar_t
 {
   wchar_t ch_enc{L'\0'};
 
